/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.model.validation;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.PARAMETER;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.math.BigDecimal;

import javax.validation.Constraint;
import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import javax.validation.Payload;

@Documented
@Constraint(validatedBy = BigDec.Validator.class)
@Target({METHOD, FIELD, PARAMETER})
@Retention(RetentionPolicy.RUNTIME)
public @interface BigDec
{
    boolean required() default true;

    String message() default "should be 15 digits maximum";

    Class< ? >[] groups() default {};

    Class< ? extends Payload>[] payload() default {};

    static class Validator implements ConstraintValidator<BigDec, BigDecimal>
    {
        BigDec bigDec;

        @Override
        public void initialize(final BigDec constraintAnnotation)
        {
            this.bigDec = constraintAnnotation;
        }

        @Override
        public boolean isValid(final BigDecimal value, final ConstraintValidatorContext context)
        {

            if (!bigDec.required() && value == null)
            {
                return true;
            }

            boolean valid = false;

            if (value != null)
            {
                valid = value.toString().length() <= 15;
                if (!valid)
                {
                    context.disableDefaultConstraintViolation();
                    context.buildConstraintViolationWithTemplate(bigDec.message())
                        .addConstraintViolation();
                }
            }
            return valid;
        }
    }
}
