/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
/**
 * 
 */
package com.abiquo.model.validation;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.PARAMETER;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import javax.validation.Constraint;
import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import javax.validation.Payload;

import org.apache.commons.lang.StringUtils;

import com.abiquo.model.enumerator.HypervisorType;

/**
 * @author jdevesa
 */
@Documented
@Constraint(validatedBy = Hypervisor.Validator.class)
@Target({METHOD, FIELD, PARAMETER})
@Retention(RetentionPolicy.RUNTIME)
public @interface Hypervisor
{
    boolean required() default true;

    String message() default "must be one of the following values { VBOX, KVM, XEN_3, VMX_04, HYPERV_301, XENSERVER, vbox, kvm, xen_3, vmx_04, hyperv_301, xenserver}";

    Class< ? >[] groups() default {};

    Class< ? extends Payload>[] payload() default {};

    static class Validator implements ConstraintValidator<Hypervisor, String>
    {
        Hypervisor hyp;

        @Override
        public void initialize(final Hypervisor constraintAnnotation)
        {
            this.hyp = constraintAnnotation;
        }

        @Override
        public boolean isValid(final String value, final ConstraintValidatorContext context)
        {
            if (!hyp.required() && StringUtils.isEmpty(value))
            {
                return true;
            }

            boolean valid = false;
            if (!StringUtils.isEmpty(value))
            {
                for (HypervisorType currentHyp : HypervisorType.values())
                {
                    if (currentHyp.getValue().equalsIgnoreCase(value))
                    {
                        valid = true;
                        break;
                    }
                }
            }

            if (!valid)
            {
                context.disableDefaultConstraintViolation();
                context.buildConstraintViolationWithTemplate(hyp.message())
                    .addConstraintViolation();
            }

            return valid;
        }

    }
}
