/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
/**
 * 
 */
package com.abiquo.model.validation;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.PARAMETER;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import javax.validation.Constraint;
import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import javax.validation.Payload;

import org.apache.commons.lang.StringUtils;

import com.abiquo.model.enumerator.StatefulInclusion;

/**
 * @author scastro
 */
@Documented
@Constraint(validatedBy = IncludeStateful.Validator.class)
@Target({METHOD, FIELD, PARAMETER})
@Retention(RetentionPolicy.RUNTIME)
public @interface IncludeStateful
{
    boolean required() default true;

    String message() default "must be one of the following values { ALL, USED, NOTUSED, all, used, notused }";

    Class< ? >[] groups() default {};

    Class< ? extends Payload>[] payload() default {};

    static class Validator implements ConstraintValidator<IncludeStateful, String>
    {
        IncludeStateful type;

        @Override
        public void initialize(final IncludeStateful constraintAnnotation)
        {
            this.type = constraintAnnotation;
        }

        @Override
        public boolean isValid(final String value, final ConstraintValidatorContext context)
        {
            if (!type.required() && StringUtils.isEmpty(value))
            {
                return true;
            }

            boolean valid = false;
            if (!StringUtils.isEmpty(value))
            {
                for (StatefulInclusion currentType : StatefulInclusion.values())
                {
                    if (currentType.name().equalsIgnoreCase(value))
                    {
                        valid = true;
                        break;
                    }
                }
            }

            if (!valid)
            {
                context.disableDefaultConstraintViolation();
                context.buildConstraintViolationWithTemplate(type.message())
                    .addConstraintViolation();
            }

            return valid;
        }

    }
}
