/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.model.validation;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.PARAMETER;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import javax.validation.Constraint;
import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import javax.validation.Payload;

import org.apache.commons.lang.StringUtils;

import com.abiquo.model.util.AddressingUtils;

@Documented
@Constraint(validatedBy = IscsiPath.Validator.class)
@Target({METHOD, FIELD, PARAMETER})
@Retention(RetentionPolicy.RUNTIME)
public @interface IscsiPath
{
    public static final String ERROR_MESSAGE =
        "must be a valid iSCSI path. Check that the IQN and LUN are well formed.";

    boolean required() default true;

    String message() default ERROR_MESSAGE;

    Class< ? >[] groups() default {};

    Class< ? extends Payload>[] payload() default {};

    static class Validator implements ConstraintValidator<IscsiPath, String>
    {
        IscsiPath iscsiPath;

        @Override
        public void initialize(final IscsiPath constraintAnnotation)
        {
            this.iscsiPath = constraintAnnotation;
        }

        @Override
        public boolean isValid(final String value, final ConstraintValidatorContext context)
        {
            if (!iscsiPath.required() && StringUtils.isEmpty(value))
            {
                return true;
            }

            boolean valid = !StringUtils.isEmpty(value) && AddressingUtils.isValidPath(value);

            if (!valid)
            {
                context.disableDefaultConstraintViolation();
                context.buildConstraintViolationWithTemplate(iscsiPath.message())
                    .addConstraintViolation();
            }

            return valid;
        }

    }
}
