/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.model.validation;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.PARAMETER;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import javax.validation.Constraint;
import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import javax.validation.Payload;

@Documented
@Constraint(validatedBy = Port.Validator.class)
@Target( {METHOD, FIELD, PARAMETER})
@Retention(RetentionPolicy.RUNTIME)
public @interface Port
{
    public static final int MAX_VALUE = 65535;

    public static final int MIN_VALUE = 0;

    boolean required() default true;

    String message() default "invalid port :";

    Class< ? >[] groups() default {};

    Class< ? extends Payload>[] payload() default {};

    static class Validator implements ConstraintValidator<Port, Integer>
    {
        Port port;

        @Override
        public void initialize(final Port constraintAnnotation)
        {
            this.port = constraintAnnotation;
        }

        @Override
        public boolean isValid(final Integer value, final ConstraintValidatorContext context)
        {
            if (!port.required() && value == null)
            {
                return true;
            }

            if (port.required() && value == null)
            {
                return false;
            }

            Boolean valid = value >= MIN_VALUE && value <= MAX_VALUE;

            if (!valid)
            {
                context.disableDefaultConstraintViolation();
                context.buildConstraintViolationWithTemplate(port.message() + value)
                    .addConstraintViolation();
            }

            return valid;
        }

    }
}
