/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.model.validation;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.PARAMETER;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import javax.validation.Constraint;
import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import javax.validation.Payload;

import org.apache.commons.lang.StringUtils;

import com.abiquo.model.util.ChefUtils;

@Documented
@Constraint(validatedBy = RunlistItem.Validator.class)
@Target({METHOD, FIELD, PARAMETER})
@Retention(RetentionPolicy.RUNTIME)
public @interface RunlistItem
{
    public static final String ERROR_MESSAGE = "must be a valid runlist element";

    boolean required() default true;

    String message() default ERROR_MESSAGE;

    Class< ? >[] groups() default {};

    Class< ? extends Payload>[] payload() default {};

    static class Validator implements ConstraintValidator<RunlistItem, String>
    {
        RunlistItem element;

        @Override
        public void initialize(final RunlistItem constraintAnnotation)
        {
            this.element = constraintAnnotation;
        }

        @Override
        public boolean isValid(final String value, final ConstraintValidatorContext context)
        {
            if (!element.required() && StringUtils.isEmpty(value))
            {
                return true;
            }

            boolean valid =
                !StringUtils.isEmpty(value)
                    && (ChefUtils.isRecipe(value) || ChefUtils.isRole(value));

            if (!valid)
            {
                context.disableDefaultConstraintViolation();
                context.buildConstraintViolationWithTemplate(element.message())
                    .addConstraintViolation();
            }

            return valid;
        }
    }
}
