/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.model.validation;

import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.PARAMETER;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import javax.validation.Constraint;
import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import javax.validation.Payload;

import com.abiquo.server.core.infrastructure.Rack.VlanRange;

@Documented
@Constraint(validatedBy = VlanIdRange.Validator.class)
@Target({METHOD, FIELD, PARAMETER})
@Retention(RetentionPolicy.RUNTIME)
public @interface VlanIdRange
{
    boolean required() default true;

    String message() default "is invalid. Max VLAN id must be greater or equal than min VLAN id";

    Class< ? >[] groups() default {};

    Class< ? extends Payload>[] payload() default {};

    static class Validator implements ConstraintValidator<VlanIdRange, VlanRange>
    {
        VlanIdRange vlanIdRange;

        @Override
        public void initialize(final VlanIdRange constraintAnnotation)
        {
            this.vlanIdRange = constraintAnnotation;
        }

        @Override
        public boolean isValid(final VlanRange value, final ConstraintValidatorContext context)
        {
            boolean valid = value.vlanIdMin <= value.vlanIdMax;

            if (!valid)
            {
                context.disableDefaultConstraintViolation();
                context.buildConstraintViolationWithTemplate(vlanIdRange.message())
                    .addConstraintViolation();
            }

            return valid;
        }
    }

}
