/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.appslibrary;

import javax.persistence.EntityManager;

import org.hibernate.Criteria;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.common.persistence.DefaultDAOBase;
import com.abiquo.server.core.enterprise.Enterprise;

@Repository("jpaAppsLibraryDAO")
public class AppsLibraryDAO extends DefaultDAOBase<Integer, AppsLibrary>
{
    public AppsLibraryDAO()
    {
        super(AppsLibrary.class);
    }

    public AppsLibraryDAO(final EntityManager entityManager)
    {
        super(AppsLibrary.class, entityManager);
    }

    public AppsLibrary findByEnterprise(final Enterprise enterprise)
    {
        Criteria criteria = createCriteria(sameEnterprise(enterprise));
        criteria.addOrder(Order.asc(AppsLibrary.ENTERPRISE_PROPERTY));

        return (AppsLibrary) criteria.uniqueResult();
    }

    public AppsLibrary findByEnterpriseOrInitialize(final Enterprise enterprise)
    {
        AppsLibrary appsLib = findByEnterprise(enterprise);

        if (appsLib == null)
        {
            appsLib = new AppsLibrary(enterprise);
            persist(appsLib);
        }

        return appsLib;
    }

    private static Criterion sameEnterprise(final Enterprise enterprise)
    {
        return Restrictions.eq(AppsLibrary.ENTERPRISE_PROPERTY, enterprise);
    }

}
