/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.appslibrary;

import java.util.List;

import javax.persistence.EntityManager;

import org.apache.commons.collections.CollectionUtils;
import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.common.persistence.DefaultDAOBase;

@Repository("jpaTemplateDefinitionDAO")
public class TemplateDefinitionDAO extends DefaultDAOBase<Integer, TemplateDefinition>
{

    private final static String FIND_BY_ENTERPRISE =
        "SELECT temDef FROM TemplateDefinition temDef " //
            + "WHERE temDef.appsLibrary.enterprise.id = :enterpriseId ";

    public TemplateDefinitionDAO()
    {
        super(TemplateDefinition.class);
    }

    public TemplateDefinitionDAO(final EntityManager entityManager)
    {
        super(TemplateDefinition.class, entityManager);
    }

    public TemplateDefinition findByUrl(final String url)
    {
        List<TemplateDefinition> defs = getResultList(createCriteria(sameUrl(url)));

        return CollectionUtils.isEmpty(defs) ? null : defs.get(0);
    }

    @SuppressWarnings("unchecked")
    public List<TemplateDefinition> findByEnterprise(final Integer enterpriseId)
    {
        Query query = getSession().createQuery(FIND_BY_ENTERPRISE);
        query.setParameter("enterpriseId", enterpriseId);
        return query.list();
    }

    public List<TemplateDefinition> findByAppsLibrary(final AppsLibrary appsLibrary)
    {
        Criteria criteria = createCriteria(sameAppsLibrary(appsLibrary));
        criteria.addOrder(Order.asc(TemplateDefinition.NAME_PROPERTY));
        return getResultList(criteria);
    }

    private static Criterion sameUrl(final String url)
    {
        return Restrictions.eq(TemplateDefinition.URL_PROPERTY, url);
    }

    private static Criterion sameAppsLibrary(final AppsLibrary appsLibrary)
    {
        return Restrictions.eq(TemplateDefinition.APPS_LIBRARY_PROPERTY, appsLibrary);
    }
}
