/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.appslibrary;

import java.util.List;

import javax.persistence.EntityManager;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.common.persistence.DefaultDAOBase;
import com.abiquo.server.core.enterprise.Enterprise;

@Repository("jpaTemplateDefinitionListDAO")
public class TemplateDefinitionListDAO extends DefaultDAOBase<Integer, TemplateDefinitionList>
{
    private final static String FIND_BY_ENTERPRISE =
        "SELECT temDeflist FROM TemplateDefinitionList temDeflist " //
            + "WHERE temDeflist.appsLibrary.enterprise.id = :enterpriseId ";

    private final static String FIND_BY_NAME_AND_ENTERPRISE =
        "SELECT temDeflist FROM TemplateDefinitionList temDeflist " //
            + "WHERE temDeflist.appsLibrary.enterprise.id = :enterpriseId and temDeflist.name = :nameEnt";

    public TemplateDefinitionListDAO()
    {
        super(TemplateDefinitionList.class);
    }

    public TemplateDefinitionListDAO(final EntityManager entityManager)
    {
        super(TemplateDefinitionList.class, entityManager);
    }

    @SuppressWarnings("unchecked")
    public List<TemplateDefinitionList> findByEnterprise(final Integer enterpriseId)
    {
        Query query = getSession().createQuery(FIND_BY_ENTERPRISE);
        query.setParameter("enterpriseId", enterpriseId);

        return query.list();
    }

    public TemplateDefinitionList findByNameAndEnterprise(final String nameEnt, final Enterprise ent)
    {
        Query query = getSession().createQuery(FIND_BY_NAME_AND_ENTERPRISE);
        query.setParameter("enterpriseId", ent.getId());
        query.setParameter("nameEnt", nameEnt);

        return (TemplateDefinitionList) query.uniqueResult();
    }

    @SuppressWarnings("unchecked")
    public TemplateDefinitionList findByName(final String name)
    {
        Criteria criteria = createCriteria(sameName(name));
        criteria.addOrder(Order.asc(TemplateDefinitionList.NAME_PROPERTY));
        return (TemplateDefinitionList) criteria.uniqueResult();
    }

    private static Criterion sameName(final String name)
    {
        return Restrictions.eq(TemplateDefinitionList.NAME_PROPERTY, name);
    }
}
