/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.appslibrary;

import java.util.List;

import javax.persistence.EntityManager;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.common.DefaultRepBase;
import com.abiquo.server.core.enterprise.Enterprise;

@Repository
public class TemplateDefinitionRep extends DefaultRepBase
{

    @Autowired
    TemplateDefinitionDAO dao;

    @Autowired
    TemplateDefinitionListDAO listDao;

    @Autowired
    AppsLibraryDAO appsLibraryDao;

    @Autowired
    CategoryDAO categoryDao;

    public TemplateDefinitionRep()
    {

    }

    public TemplateDefinitionRep(final EntityManager em)
    {
        assert entityManager != null;
        assert entityManager.isOpen();

        this.entityManager = em;

        dao = new TemplateDefinitionDAO(em);
        listDao = new TemplateDefinitionListDAO(em);
        appsLibraryDao = new AppsLibraryDAO(em);
        categoryDao = new CategoryDAO(em);
    }

    public List<TemplateDefinition> getTemplateDefinitionsByEnterprise(final Integer idEnterprise)
    {
        return dao.findByEnterprise(idEnterprise);
    }

    public TemplateDefinition addTemplateDefinition(final TemplateDefinition templateDef,
        final Enterprise enterprise)
    {
        AppsLibrary appsLib = appsLibraryDao.findByEnterprise(enterprise);

        Category category = categoryDao.findById(templateDef.getCategory().getId());
        if (category == null)
        {
            categoryDao.persist(templateDef.getCategory());
        }

        templateDef.setAppsLibrary(appsLib);
        dao.persist(templateDef);
        dao.flush();

        return templateDef;
    }

    public TemplateDefinition getTemplateDefinition(final Integer id)
    {
        return dao.findById(id);
    }

    public TemplateDefinition updateTemplateDefinition(final Integer templateUrl,
        final TemplateDefinition templateDef, final Enterprise enterprise)
    {
        TemplateDefinition old = dao.findById(templateUrl);

        // TODO - Apply changes and compare etags
        old.setName(templateDef.getName());
        old.setDescription(templateDef.getDescription());

        AppsLibrary appsLib = appsLibraryDao.findByEnterprise(enterprise);
        templateDef.setAppsLibrary(appsLib);
        old.setId(templateUrl);
        old.setCategory(templateDef.getCategory());
        old.setType(templateDef.getType());
        old.setIconUrl(templateDef.getIconUrl());
        old.setProductName(templateDef.getProductName());
        old.setProductUrl(templateDef.getProductUrl());
        old.setProductVendor(templateDef.getProductVendor());
        old.setProductVersion(templateDef.getProductVersion());
        old.setUrl(templateDef.getUrl());
        old.setLoginPassword(templateDef.getLoginPassword());
        old.setLoginUser(templateDef.getLoginUser());
        old.setOsType(templateDef.getOsType());
        old.setOsVersion(templateDef.getOsVersion());
        old.setEthernetDriverType(templateDef.getEthernetDriverType());
        old.setDiskControllerType(templateDef.getDiskControllerType());
        old.setTemplateDefinitionLists(templateDef.getTemplateDefinitionLists());

        return old;
    }

    public void removeTemplateDefinition(final Integer id)
    {
        TemplateDefinition ovfPackage = dao.findById(id);

        // manually remove lists associated
        // As OVFPackage<->OVFPackageLists is a Many-to-many relation, the delete operation
        // must be done manually for the dependant (not owner) side in the relation: OVFPackage in
        // this case
        List<TemplateDefinitionList> lists = ovfPackage.getTemplateDefinitionLists();
        for (TemplateDefinitionList templateList : lists)
        {
            templateList.getTemplateDefinitions().remove(ovfPackage);
            listDao.flush();
        }

        dao.remove(ovfPackage);
    }

    public void removeTemplateDefinitionList(final TemplateDefinitionList templateDefList)
    {
        for (TemplateDefinition ovf : templateDefList.getTemplateDefinitions())
        {
            if (ovf.getTemplateDefinitionLists().size() == 1)
            {
                dao.remove(ovf);
            }
            else
            {
                ovf.getTemplateDefinitionLists().remove(templateDefList);
            }
        }
        listDao.remove(templateDefList);

    }

    public List<TemplateDefinitionList> getTemplateDefinitionLists()
    {
        return listDao.findAll();
    }

    public TemplateDefinitionList findTemplateDefinitionListByNameAndEnterprise(final String name,
        final Enterprise ent)
    {

        return listDao.findByNameAndEnterprise(name, ent);
    }

    public void persistTemplateDefinitionList(final TemplateDefinitionList templateDefList)
    {
        listDao.persist(templateDefList);
    }

    public void updateTemplateDefinitionList(final TemplateDefinitionList templateDefList)
    {
        listDao.flush();
    }

    public TemplateDefinitionList getTemplateDefinitionList(final Integer id)
    {
        return listDao.findById(id);
    }

    public List<TemplateDefinitionList> getTemplateDefinitionListsByEnterprise(
        final Integer idEnterprise)
    {
        return listDao.findByEnterprise(idEnterprise);

    }
}
