/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.cloud;

public enum State
{
    ON, OFF, PAUSED, ALLOCATED, CONFIGURED, UNKNOWN, NOT_ALLOCATED, LOCKED;

    public static State fromValue(final String value)
    {
        return State.valueOf(value.toUpperCase());
    }

    public State travel(final StateTransition transaction)
    {
        if (!transaction.isValidOrigin(this))
        {
            throw new RuntimeException("Invalid origin " + this + " for transaction " + transaction);
        }

        return transaction.getEndState();
    }

    public int id()
    {
        return ordinal() + 1;
    }

    public static State fromId(final int id)
    {
        return State.values()[id - 1];
    }

    public String toOVF()
    {
        switch (this)
        {
            case ON:
                return "POWERUP_ACTION";
            case PAUSED:
                return "PAUSE_ACTION";
            case OFF:
                return "POWERDOWN_ACTION";
        }
        return null;
    }

    public String toResourceState()
    {
        switch (this)
        {
            case ON:
                return "PowerUp";
            case PAUSED:
                return "Pause";
            case OFF:
                return "PowerOff";
                // case REBOOTED:
                // return "Resume";
        }
        return null;
    }

    @Override
    public String toString()
    {
        return "";
    }

    public boolean isDeployed()
    {
        switch (this)
        {
            case ON:
            case OFF:
            case PAUSED:
                return true;
            // Configured state. The VM is in the hypervisor but it
            // has never been powered on. The Chef agent has not run
            // yet and the node does not exist in the Chef server.
            case CONFIGURED:
            default:
                return false;
        }
    }
}
