/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.cloud.chef;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.hibernate.Criteria;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.cloud.VirtualMachine;
import com.abiquo.server.core.common.persistence.DefaultDAOBase;

@Repository("jpaRunlistElementDAO")
public class RunlistElementDAO extends DefaultDAOBase<Integer, RunlistElement>
{
    public RunlistElementDAO()
    {
        super(RunlistElement.class);
    }

    public RunlistElementDAO(final EntityManager entityManager)
    {
        super(RunlistElement.class, entityManager);
    }

    public List<RunlistElement> findByVirtualMachine(final VirtualMachine virtualmachine)
    {
        Criteria criteria = createCriteria(sameVirtualMachine(virtualmachine));
        criteria.addOrder(Order.asc(RunlistElement.PRIORITY_PROPERTY));
        return getResultList(criteria);
    }

    public void clearVirtualMachineRunlist(final VirtualMachine virtualmachine)
    {
        Query query =
            getEntityManager().createQuery(
                "delete from " + getPersistentClass().getName()
                    + " where virtualMachine = :virtualMachine");

        query.setParameter("virtualMachine", virtualmachine);
        query.executeUpdate();
    }

    private static Criterion sameVirtualMachine(final VirtualMachine virtualmachine)
    {
        return Restrictions.eq(RunlistElement.VIRTUALMACHINE_PROPERTY, virtualmachine);
    }
}
