/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.common;

import java.io.Serializable;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Set;

import javax.persistence.MappedSuperclass;
import javax.validation.ConstraintViolation;
import javax.validation.Validation;

import com.abiquo.model.transport.error.CommonError;
import com.softwarementors.bzngine.entities.PersistentVersionedEntityBase;
import com.softwarementors.validation.ValidationManager;

@MappedSuperclass
public abstract class MyGenericEntityBase<T extends Serializable> extends
    PersistentVersionedEntityBase<T>
{
    protected MyGenericEntityBase()
    {
        super();
    }

    public boolean isValid()
    {
        return Validation.buildDefaultValidatorFactory().getValidator().validate(this).isEmpty();
    }

    public Set<CommonError> getValidationErrors()
    {
        Set<ConstraintViolation< ? extends Object>> violations =
            new LinkedHashSet<ConstraintViolation< ? extends Object>>(ValidationManager
                .getValidator().validate(this));

        Set<CommonError> errors = new HashSet<CommonError>();

        for (ConstraintViolation< ? extends Object> constrain : violations)
        {
            errors.add(buildCommonError(constrain));
        }

        return errors;
    }

    private CommonError buildCommonError(final ConstraintViolation< ? extends Object> constraint)
    {
        String code =
            String.format("CONSTR-%s", constraint.getConstraintDescriptor().getAnnotation()
                .annotationType().getSimpleName().toUpperCase());

        String message =
            String.format("The property '%s' %s.", constraint.getPropertyPath().toString(),
                constraint.getMessage());

        return new CommonError(code, message);
    }
}
