/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.config;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Table;

import org.hibernate.validator.constraints.Length;

import com.abiquo.server.core.common.DefaultEntityBase;
import com.softwarementors.validation.constraints.LeadingOrTrailingWhitespace;
import com.softwarementors.validation.constraints.Required;

@Entity
@Table(name = License.TABLE_NAME)
@org.hibernate.annotations.Table(appliesTo = License.TABLE_NAME)
public class License extends DefaultEntityBase
{
    // *************************** Decrypted license field constants ***********************

    /** The date format used to store license dates. */
    public static final String DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";

    /** The Number of Cores license key. */
    public static final String NUMBER_OF_CORES_KEY = "license.cores.max";

    /** The Expiration Date license key. */
    public static final String EXPIRATION_DATE_KEY = "license.expires";

    /** The enabled plugins. */
    public static final String ENABLED_PLUGINS = "license.plugins";

    // *************************** License model ***********************

    public static final String TABLE_NAME = "license";

    protected License()
    {
    }

    private final static String ID_COLUMN = "idLicense";

    @Id
    @GeneratedValue
    @Column(name = ID_COLUMN, nullable = false)
    private Integer id;

    @Override
    public Integer getId()
    {
        return this.id;
    }

    public final static String DATA_PROPERTY = "data";

    private final static boolean DATA_REQUIRED = true;

    /* package */final static int DATA_LENGTH_MIN = 1;

    private final static boolean DATA_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    private final static String DATA_COLUMN = "data";

    @Column(name = DATA_COLUMN, nullable = !DATA_REQUIRED, columnDefinition = "TEXT")
    private String data;

    @Required(value = DATA_REQUIRED)
    @Length(min = DATA_LENGTH_MIN)
    @LeadingOrTrailingWhitespace(allowed = DATA_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getData()
    {
        return this.data;
    }

    public void setData(final String data)
    {
        this.data = data;
    }

    // *************************** Mandatory constructors ***********************

    public License(final String data)
    {
        super();
        this.data = data;
    }

}
