/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.config;

import java.util.List;

import javax.persistence.EntityManager;

import org.hibernate.Criteria;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.common.persistence.DefaultDAOBase;

@Repository("jpaSystemPropertyDAO")
public class SystemPropertyDAO extends DefaultDAOBase<Integer, SystemProperty>
{
    public SystemPropertyDAO()
    {
        super(SystemProperty.class);
    }

    public SystemPropertyDAO(EntityManager entityManager)
    {
        super(SystemProperty.class, entityManager);
    }

    public boolean existsAnyWithName(String name)
    {
        return existsAnyByCriterions(equalsName(name));
    }

    public boolean existsAnyOtherWithName(SystemProperty systemProperty, String name)
    {
        return this.existsAnyOtherByCriterions(systemProperty, equalsName(name));
    }

    public SystemProperty findByName(String name)
    {
        Criteria criteria = createCriteria(equalsName(name));
        criteria.addOrder(Order.asc(SystemProperty.NAME_PROPERTY));
        List<SystemProperty> result = criteria.list();
        return result.isEmpty() ? null : result.get(0);
    }

    public List<SystemProperty> findByComponent(String component)
    {
        Criteria criteria = createCriteria(equalsComponent(component));
        criteria.addOrder(Order.asc(SystemProperty.NAME_PROPERTY));
        return getResultList(criteria);
    }

    private Criterion equalsName(String name)
    {
        return Restrictions.eq(SystemProperty.NAME_PROPERTY, name);
    }

    private Criterion equalsComponent(String component)
    {
        String prefix = component + ".";
        return Restrictions.like(SystemProperty.NAME_PROPERTY, prefix, MatchMode.START);
    }

}
