/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.enterprise;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.hibernate.annotations.ForeignKey;

import com.abiquo.server.core.common.DefaultEntityBase;
import com.abiquo.server.core.infrastructure.storage.Tier;
import com.softwarementors.validation.constraints.Required;

@Entity
@Table(name = AllowedTier.TABLE_NAME)
@org.hibernate.annotations.Table(appliesTo = AllowedTier.TABLE_NAME)
public class AllowedTier extends DefaultEntityBase
{
    public static final String TABLE_NAME = "allowed_tier_by_enterprise";

    protected AllowedTier()
    {
    }

    /**
     * Public constructor with the mandatory attributes
     * 
     * @param datacenterLimit Limit who relates the datacenter owner of the tier and the enterprise
     *            where allow the tier
     * @param tier Tier to be allowed
     */
    public AllowedTier(final DatacenterLimits datacenterLimit, final Tier tier)
    {
        this.datacenterLimit = datacenterLimit;
        this.tier = tier;
    }

    public final static String ID_COLUMN = "id";

    @Id
    @GeneratedValue
    @Column(name = ID_COLUMN, nullable = false)
    private Integer id;

    @Override
    public Integer getId()
    {
        return this.id;
    }

    public final static String TIER_PROPERTY = "tier";

    private final static boolean TIER_REQUIRED = true;

    private final static String TIER_ID_COLUMN = "idTier";

    @JoinColumn(name = TIER_ID_COLUMN)
    @ManyToOne(fetch = FetchType.LAZY)
    @ForeignKey(name = "FK_" + TABLE_NAME + "_tier")
    private Tier tier;

    @Required(value = TIER_REQUIRED)
    public Tier getTier()
    {
        return this.tier;
    }

    public void setTier(final Tier tier)
    {
        this.tier = tier;
    }

    public final static String DATACENTER_LIMIT_PROPERTY = "datacenterLimit";

    private final static boolean DATACENTER_LIMIT_REQUIRED = true;

    private final static String DATACENTER_LIMIT_ID_COLUMN = "idDatacenterLimit";

    @JoinColumn(name = DATACENTER_LIMIT_ID_COLUMN)
    @ManyToOne(fetch = FetchType.LAZY)
    @ForeignKey(name = "FK_" + TABLE_NAME + "_datacenter_limit")
    private DatacenterLimits datacenterLimit;

    @Required(value = DATACENTER_LIMIT_REQUIRED)
    public DatacenterLimits getDatacenterLimit()
    {
        return this.datacenterLimit;
    }

    public void setDatacenterLimit(final DatacenterLimits datacenterLimits)
    {
        this.datacenterLimit = datacenterLimits;
    }

}
