/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.enterprise;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Table;

import org.hibernate.validator.constraints.Length;

import com.abiquo.server.core.common.DefaultEntityBase;
import com.softwarementors.validation.constraints.LeadingOrTrailingWhitespace;
import com.softwarementors.validation.constraints.Required;

@Entity
@Table(name = EnterpriseTheme.TABLE_NAME)
public class EnterpriseTheme extends DefaultEntityBase
{

    public static final String TABLE_NAME = "enterprise_theme";

    private final static String ID_COLUMN = "idEnterprise";

    protected EnterpriseTheme()
    {
        // Just for JPA support
    }

    @Id
    @Column(name = ID_COLUMN, nullable = false)
    private Integer id;

    @Override
    public Integer getId()
    {
        return this.id;
    }

    // public final static String ENTERPRISE_PROPERTY = "enterprise";
    //
    // private final static boolean ENTERPRISE_REQUIRED = true;
    //
    // @OneToOne(optional = !ENTERPRISE_REQUIRED, fetch = FetchType.LAZY, mappedBy = "theme")
    // // @PrimaryKeyJoinColumn(name = ID_COLUMN, referencedColumnName = ID_COLUMN)
    // private Enterprise enterprise;
    //
    // @Required(value = ENTERPRISE_REQUIRED)
    // public Enterprise getEnterprise()
    // {
    // return this.enterprise;
    // }
    //
    // public void setEnterprise(final Enterprise enterprise)
    // {
    // this.enterprise = enterprise;
    // }

    public final static String THEME_PROPERTY = "theme";

    private final static boolean THEME_REQUIRED = false;

    /* package */final static int THEME_LENGTH_MIN = 0;

    private final static boolean THEME_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    private final static String THEME_COLUMN = "theme";

    @Column(name = THEME_COLUMN, columnDefinition = "TEXT", nullable = !THEME_REQUIRED)
    private String theme;

    @Required(value = THEME_REQUIRED)
    @Length(min = THEME_LENGTH_MIN)
    @LeadingOrTrailingWhitespace(allowed = THEME_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getTheme()
    {
        return this.theme;
    }

    public void setTheme(final String theme)
    {
        this.theme = theme;
    }

    public final static String LOGO_PATH_PROPERTY = "company_logo_path";

    private final static boolean LOGO_PATH_REQUIRED = false;

    /* package */final static int LOGO_PATH_LENGTH_MIN = 0;

    private final static boolean LOGO_PATH_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    private final static String LOGO_PATH_COLUMN = "company_logo_path";

    @Column(name = LOGO_PATH_COLUMN, columnDefinition = "TEXT", nullable = !LOGO_PATH_REQUIRED)
    private String company_logo_path;

    @Required(value = LOGO_PATH_REQUIRED)
    @Length(min = LOGO_PATH_LENGTH_MIN)
    @LeadingOrTrailingWhitespace(allowed = LOGO_PATH_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getLogo()
    {
        return this.company_logo_path;
    }

    public void setLogo(final String company_logo_path)
    {
        this.company_logo_path = company_logo_path;
    }

    // ---------------Constructor-----------------//

    public EnterpriseTheme(final String theme, final String logoPath)
    {
        setTheme(theme);
        setLogo(logoPath);
    }

    // only for test support
    public EnterpriseTheme(final String theme2, final String logoPath, final Enterprise enterprise)
    {
        setTheme(theme);
        setLogo(logoPath);
        // setEnterprise(enterprise);
    }

    /**
     * This entity doesn't have generated value for the id, for any creation,update, get or delete
     * must be done through the enterprise dao.
     * 
     * @param id
     */
    public void setId(final Integer id)
    {
        this.id = id;

    }
}
