/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.enterprise;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.common.persistence.DefaultDAOBase;
import com.abiquo.server.core.enterprise.Scope.OrderByEnum;
import com.abiquo.server.core.util.PagedList;

@Repository("jpaScopeDAO")
public class ScopeDAO extends DefaultDAOBase<Integer, Scope>
{
    public ScopeDAO()
    {
        super(Scope.class);
    }

    public ScopeDAO(final EntityManager entityManager)
    {
        super(Scope.class, entityManager);
    }

    public static final String FIND_ALL = "SELECT s FROM Scope s WHERE (s.name LIKE :filterLike)";

    public List<Scope> findAll(final String has, final Boolean asc, final OrderByEnum orderByEnum,
        final Integer limit, final Integer startwith)
    {
        Map<String, String> params = new HashMap<String, String>();
        params.put("filterLike", has == null || has.isEmpty() ? "%" : "%" + has + "%");

        Query finalQuery = getSession().createQuery(FIND_ALL + defineOrderBy(orderByEnum, asc));
        finalQuery.setProperties(params);

        Integer totalResults = count(finalQuery, params).intValue();
        Integer start = startwith;

        finalQuery.setMaxResults(limit);
        if (limit == 0)
        {
            finalQuery.setMaxResults(totalResults);
        }
        else if (limit != null)
        {
            finalQuery.setMaxResults(limit);
        }

        if (startwith >= totalResults)
        {
            start = totalResults - limit;
        }
        finalQuery.setFirstResult(start);

        PagedList<Scope> scopes = new PagedList<Scope>(finalQuery.list());
        scopes.setTotalResults(totalResults);
        scopes.setCurrentElement(start);
        scopes.setPageSize(limit);

        return scopes;
    }

    private String defineOrderBy(final Scope.OrderByEnum orderBy, final Boolean asc)
    {
        StringBuilder queryString = new StringBuilder();

        queryString.append(" order by ");

        switch (orderBy)
        {
            case NAME:
                queryString.append("s.name ");
                break;
        }

        if (asc)
        {
            queryString.append("asc");
        }
        else
        {
            queryString.append("desc");
        }
        return queryString.toString();
    }

    private Criterion sameName(final String name)
    {
        assert name != null;

        return Restrictions.eq(Scope.NAME_PROPERTY, name);
    }

    public Boolean existsByName(final String name)
    {
        return existsAnyByCriterions(sameName(name));
    }

    public Scope findScopeByName(final String name)
    {
        Criteria criteria = createCriteria(sameName(name));
        return getSingleResult(criteria);

    }
}
