/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.enterprise;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.hibernate.annotations.ForeignKey;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.Range;

import com.abiquo.server.core.common.DefaultEntityBase;
import com.softwarementors.validation.constraints.LeadingOrTrailingWhitespace;
import com.softwarementors.validation.constraints.Required;

@Entity
@Table(name = ScopeEntity.TABLE_NAME)
@org.hibernate.annotations.Table(appliesTo = ScopeEntity.TABLE_NAME)
public class ScopeEntity extends DefaultEntityBase
{
    public static final String TABLE_NAME = "scope_resource";

    // DO NOT ACCESS: present due to needs of infrastructure support. *NEVER* call from business
    // code
    protected ScopeEntity()
    {
        // Just for JPA support
    }

    private final static String ID_COLUMN = "id";

    @Id
    @GeneratedValue
    @Column(name = ID_COLUMN, nullable = false)
    private Integer id;

    public Integer getId()
    {
        return this.id;
    }

    public final static String ID_RESOURCE_PROPERTY = "idResource";

    private final static boolean ID_RESOURCE_REQUIRED = true;

    private final static String ID_RESOURCE_COLUMN = "idResource";

    private final static int ID_RESOURCE_MIN = 1;

    private final static int ID_RESOURCE_MAX = Integer.MAX_VALUE;

    @Column(name = ID_RESOURCE_COLUMN, nullable = !ID_RESOURCE_REQUIRED)
    @Range(min = ID_RESOURCE_MIN, max = ID_RESOURCE_MAX)
    private int idResource;

    public int getIdResource()
    {
        return this.idResource;
    }

    public void setIdResource(final int idResource)
    {
        this.idResource = idResource;
    }

    public final static String TYPE_PROPERTY = "type";

    private final static boolean TYPE_REQUIRED = true;

    private final static int TYPE_LENGTH_MIN = 1;

    private final static int TYPE_LENGTH_MAX = 64;

    private final static boolean TYPE_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    private final static String TYPE_COLUMN = "resourceType";

    @Column(name = TYPE_COLUMN, nullable = !TYPE_REQUIRED, length = TYPE_LENGTH_MAX)
    private String type;

    @Required(value = TYPE_REQUIRED)
    @Length(min = TYPE_LENGTH_MIN, max = TYPE_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = TYPE_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getType()
    {
        return this.type;
    }

    public void setType(final String type)
    {
        this.type = type;
    }

    public final static String SCOPE_PROPERTY = "scope";

    private final static boolean SCOPE_REQUIRED = true;

    private final static String SCOPE_ID_COLUMN = "idScope";

    @JoinColumn(name = SCOPE_ID_COLUMN)
    @ManyToOne(fetch = FetchType.LAZY)
    @ForeignKey(name = "fk_" + TABLE_NAME + "_scope")
    private Scope scope;

    @Required(value = SCOPE_REQUIRED)
    public Scope getScope()
    {
        return this.scope;
    }

    public void setScope(final Scope scope)
    {
        this.scope = scope;
    }

    // *************************** Mandatory constructors ***********************
    public ScopeEntity(final Scope scope, final Integer idResource, final String type)
    {
        setScope(scope);
        setType(type);
        setIdResource(idResource);
    }

    @Override
    public boolean equals(final Object scopeToCompare)
    {
        ScopeEntity other = (ScopeEntity) scopeToCompare;
        return other.getIdResource() == getIdResource() && other.getType().equals(getType())
            && other.getScope().getId() == getScope().getId();
    }

}
