/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.enterprise;

import java.util.List;

import javax.persistence.EntityManager;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.common.DefaultRepBase;
import com.abiquo.server.core.enterprise.Scope.OrderByEnum;

@Repository
public class ScopeRep extends DefaultRepBase
{

    @Autowired
    private ScopeDAO dao;

    @Autowired
    private ScopeEntityDAO scopeEntityDao;

    public ScopeRep()
    {

    }

    public ScopeRep(final EntityManager entityManager)
    {
        assert entityManager != null;
        assert entityManager.isOpen();

        this.entityManager = entityManager;
        dao = new ScopeDAO(entityManager);
        scopeEntityDao = new ScopeEntityDAO(entityManager);
    }

    public List<Scope> getScopes(final String has, final Boolean asc,
        final OrderByEnum orderByEnum, final Integer limit, final Integer startwith)
    {

        return dao.findAll(has, asc, orderByEnum, limit, startwith);

    }

    public boolean existsAnyByName(final String name)
    {
        return dao.existsByName(name);
    }

    public void insert(final Scope scope)
    {

        assert scope != null;
        dao.persist(scope);
        dao.flush();

    }

    public Scope findScopeById(final Integer scopeId)
    {
        return dao.findById(scopeId);
    }

    public void removeScope(final Scope scope)
    {
        dao.remove(scope);
        dao.flush();
    }

    public void update(final Scope old)
    {
        dao.flush();
    }

    public void remove(final ScopeEntity entity)
    {
        scopeEntityDao.remove(entity);
    }

    public void flushEntity()
    {
        scopeEntityDao.flush();
    }

    public void insert(final ScopeEntity ent)
    {
        assert ent != null;
        scopeEntityDao.persist(ent);
        scopeEntityDao.flush();

    }

    public List<Scope> findAll()
    {
        return dao.findAll();
    }

    public Scope findScopeByName(final String name)
    {
        return dao.findScopeByName(name);

    }
}
