/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.enterprise;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.hibernate.annotations.ForeignKey;
import org.hibernate.validator.constraints.Length;

import com.abiquo.model.enumerator.AuthType;
import com.abiquo.server.core.common.DefaultEntityBase;
import com.softwarementors.validation.constraints.LeadingOrTrailingWhitespace;
import com.softwarementors.validation.constraints.Required;

@Entity
@Table(name = Session.TABLE_NAME)
@org.hibernate.annotations.Table(appliesTo = Session.TABLE_NAME)
public class Session extends DefaultEntityBase
{
    public static final String TABLE_NAME = "session";

    protected Session()
    {
        // For JPA support
    }

    public Session(final User user, final String key, final Date expireDate)
    {
        setUser(user);
        setKey(key);
        setExpireDate(expireDate);
        setAuthType(user.getAuthType());
        setNick(user.getNick());
    }

    private final static String ID_COLUMN = "id";

    @Id
    @GeneratedValue
    @Column(name = ID_COLUMN, nullable = false)
    private Integer id;

    @Override
    public Integer getId()
    {
        return this.id;
    }

    public final static String NICK_PROPERTY = "nick";

    private final static boolean NICK_REQUIRED = true;

    public final static int NICK_LENGTH_MIN = 0;

    public final static int NICK_LENGTH_MAX = 128;

    private final static boolean NICK_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    private final static String NICK_COLUMN = "user";

    @Column(name = NICK_COLUMN, nullable = !NICK_REQUIRED, length = NICK_LENGTH_MAX)
    private String nick;

    @Required(value = NICK_REQUIRED)
    @Length(min = NICK_LENGTH_MIN, max = NICK_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = NICK_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getNick()
    {
        return this.nick;
    }

    public void setNick(final String nick)
    {
        this.nick = nick;
    }

    public final static String KEY_PROPERTY = "key";

    private final static boolean KEY_REQUIRED = true;

    public final static int KEY_LENGTH_MIN = 0;

    public final static int KEY_LENGTH_MAX = 100;

    private final static boolean KEY_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    private final static String KEY_COLUMN = "`key`"; // To avoid MySQL reserved word

    @Column(name = KEY_COLUMN, nullable = !KEY_REQUIRED, length = KEY_LENGTH_MAX)
    private String key;

    @Required(value = KEY_REQUIRED)
    @Length(min = KEY_LENGTH_MIN, max = KEY_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = KEY_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getKey()
    {
        return this.key;
    }

    public void setKey(final String key)
    {
        this.key = key;
    }

    public final static String EXPIRE_DATE_PROPERTY = "expireDate";

    private final static boolean EXPIRE_DATE_REQUIRED = true;

    private final static String EXPIRE_DATE_COLUMN = "expireDate";

    @Column(name = EXPIRE_DATE_COLUMN, nullable = !EXPIRE_DATE_REQUIRED)
    private Date expireDate;

    @Required(value = EXPIRE_DATE_REQUIRED)
    public Date getExpireDate()
    {
        return expireDate;
    }

    public void setExpireDate(final Date expireDate)
    {
        this.expireDate = expireDate;
    }

    public final static String USER_PROPERTY = "user";

    private final static boolean USER_REQUIRED = false;

    private final static String USER_ID_COLUMN = "idUser";

    @JoinColumn(name = USER_ID_COLUMN, nullable = !USER_REQUIRED)
    @ManyToOne(fetch = FetchType.LAZY)
    @ForeignKey(name = "FK_" + TABLE_NAME + "_user")
    private User user;

    @Required(value = USER_REQUIRED)
    public User getUser()
    {
        return this.user;
    }

    public void setUser(final User user)
    {
        this.user = user;
    }

    public final static String AUTH_TYPE_PROPERTY = "authType";

    private final static boolean AUTH_TYPE_REQUIRED = true;

    private final static String AUTH_TYPE_COLUMN = "authType";

    @Enumerated(value = javax.persistence.EnumType.STRING)
    @Column(name = AUTH_TYPE_COLUMN, nullable = !AUTH_TYPE_REQUIRED)
    private AuthType authType;

    @Required(value = AUTH_TYPE_REQUIRED)
    public AuthType getAuthType()
    {
        return authType;
    }

    public void setAuthType(final AuthType authType)
    {
        this.authType = authType;
    }
}
