/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.enterprise;

import java.util.Collection;
import java.util.List;

import javax.persistence.EntityManager;

import org.hibernate.Criteria;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.common.persistence.DefaultDAOBase;

@Repository("jpaSessionDAO")
public class SessionDAO extends DefaultDAOBase<Integer, Session>
{
    public SessionDAO()
    {
        super(Session.class);
    }

    public SessionDAO(final EntityManager entityManager)
    {
        super(Session.class, entityManager);
    }

    public List<Session> findByKey(final String key)
    {
        Criteria criteria = createCriteria(sameKey(key));
        return getResultList(criteria);
    }

    public Session findByUserAndKey(final User user, final String key)
    {
        Criteria criteria = createCriteria(sameKey(key), sameUser(user));
        return getSingleResult(criteria);
    }

    public static Criterion sameKey(final String key)
    {
        return Restrictions.eq(Session.KEY_PROPERTY, key);
    }

    public static Criterion sameUser(final User user)
    {
        return Restrictions.eq(Session.USER_PROPERTY, user);
    }

    public Collection<Session> findByUser(final User user)
    {
        Criteria criteria = createCriteria(sameUser(user));
        return getResultList(criteria);
    }

}
