/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.event;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Table;

import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.Range;

import com.abiquo.model.enumerator.SeverityType;
import com.abiquo.server.core.common.DefaultEntityBase;
import com.softwarementors.validation.constraints.LeadingOrTrailingWhitespace;
import com.softwarementors.validation.constraints.Required;

@Entity
@Table(name = Event.TABLE_NAME)
public class Event extends DefaultEntityBase
{
    public static final String TABLE_NAME = "metering";

    // DO NOT ACCESS: present due to needs of infrastructure support. *NEVER* call from business
    // code
    protected Event()
    {
        // Just for JPA support
    }

    public Event(final String actionPerformed, final String component, final String dc,
        final Integer idDC, final String enterprise, final Integer idEnterprise,
        final String network, final Integer idNetwork, final String performedBy,
        final String machine, final Integer idPM, final String rack, final Integer idRack,
        final SeverityType severity, final String sp, final String idSP,
        final String storageSystem, final Integer idSS, final String stacktrace,
        final String subnet, final Integer idSubnet, final String user, final Integer idUser,
        final String vapp, final Integer idVapp, final String vdc, final Integer idVDC,
        final String vm, final Integer idVM, final String volume, final String idVolume)
    {
        super();

        setActionPerformed(actionPerformed);
        setComponent(component);
        setPerformedBy(performedBy);
        setSeverity(severity);
        setStacktrace(stacktrace);
        setStorageSystem(storageSystem);
        setIdStorageSystem(idSS);
        setSubnet(subnet);
        setIdSubnet(idSubnet);
        setDatacenter(dc);
        setIdDatacenter(idDC);
        setEnterprise(enterprise);
        setIdEnterprise(idEnterprise);
        setNetwork(network);
        setIdNetwork(idNetwork);
        setPhysicalMachine(machine);
        setIdPhysicalMachine(idPM);
        setRack(rack);
        setIdRack(idRack);
        setStoragePool(sp);
        setIdStoragePool(idSP);
        setUser(user);
        setIdUser(idUser);
        setVirtualApp(vapp);
        setIdVirtualApp(idVapp);
        setVirtualDatacenter(vdc);
        setIdVirtualDatacenter(idVDC);
        setVirtualMachine(vm);
        setIdVirtualMachine(idVM);
        setVolume(volume);
        setIdVolume(idVolume);
    }

    private final static String ID_COLUMN = "idMeter";

    @Id
    @GeneratedValue
    @Column(name = ID_COLUMN, nullable = false, columnDefinition = "BIGINT")
    private Integer id;

    @Override
    public Integer getId()
    {
        return this.id;
    }

    public final static String COMPONENT_PROPERTY = "component";

    private final static boolean COMPONENT_REQUIRED = false;

    /* package */final static int COMPONENT_LENGTH_MIN = 0;

    /* package */final static int COMPONENT_LENGTH_MAX = 255;

    private final static boolean COMPONENT_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    private final static String COMPONENT_COLUMN = "component";

    @Column(name = COMPONENT_COLUMN, nullable = !COMPONENT_REQUIRED, length = COMPONENT_LENGTH_MAX)
    private String component;

    @Required(value = COMPONENT_REQUIRED)
    @Length(min = COMPONENT_LENGTH_MIN, max = COMPONENT_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = COMPONENT_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getComponent()
    {
        return this.component;
    }

    public void setComponent(final String component)
    {
        this.component = component;
    }

    public final static String ACTIONPERFORMED_PROPERTY = "actionPerformed";

    private final static boolean ACTIONPERFORMED_REQUIRED = true;

    /* package */final static int ACTIONPERFORMED_LENGTH_MIN = 0;

    /* package */final static int ACTIONPERFORMED_LENGTH_MAX = 255;

    private final static boolean ACTIONPERFORMED_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    private final static String ACTIONPERFORMED_COLUMN = "actionperformed";

    @Column(name = ACTIONPERFORMED_COLUMN, nullable = !ACTIONPERFORMED_REQUIRED, length = ACTIONPERFORMED_LENGTH_MAX)
    private String actionPerformed;

    @Required(value = ACTIONPERFORMED_REQUIRED)
    @Length(min = ACTIONPERFORMED_LENGTH_MIN, max = ACTIONPERFORMED_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = ACTIONPERFORMED_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getActionPerformed()
    {
        return this.actionPerformed;
    }

    public void setActionPerformed(final String actionPerformed)
    {
        this.actionPerformed = actionPerformed;
    }

    public final static String PERFORMEDBY_PROPERTY = "performedBy";

    private final static boolean PERFORMEDBY_REQUIRED = true;

    /* package */final static int PERFORMEDBY_LENGTH_MIN = 0;

    /* package */final static int PERFORMEDBY_LENGTH_MAX = 255;

    private final static boolean PERFORMEDBY_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    private final static String PERFORMEDBY_COLUMN = "performedby";

    @Column(name = PERFORMEDBY_COLUMN, nullable = !PERFORMEDBY_REQUIRED, length = PERFORMEDBY_LENGTH_MAX)
    private String performedBy;

    @Required(value = PERFORMEDBY_REQUIRED)
    @Length(min = PERFORMEDBY_LENGTH_MIN, max = PERFORMEDBY_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = PERFORMEDBY_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getPerformedBy()
    {
        return this.performedBy;
    }

    public void setPerformedBy(final String performedBy)
    {
        this.performedBy = performedBy;
    }

    public final static String STORAGE_POOL_PROPERTY = "storagePool";

    private final static boolean STORAGE_POOL_REQUIRED = false;

    private final static String STORAGE_POOL_COLUMN = "storagePool";

    /* package */final static int STORAGE_POOL_LENGTH_MIN = 0;

    /* package */final static int STORAGE_POOL_LENGTH_MAX = 255;

    private final static boolean STORAGE_POOL_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    @Column(name = STORAGE_POOL_COLUMN, nullable = !STORAGE_POOL_REQUIRED, length = STORAGE_POOL_LENGTH_MAX)
    private String storagePool;

    @Required(value = STORAGE_POOL_REQUIRED)
    @Length(min = STORAGE_POOL_LENGTH_MIN, max = STORAGE_POOL_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = STORAGE_POOL_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getStoragePool()
    {
        return this.storagePool;
    }

    public void setStoragePool(final String storagePool)
    {
        this.storagePool = storagePool;
    }

    public final static String STACKTRACE_PROPERTY = "stacktrace";

    private final static boolean STACKTRACE_REQUIRED = false;

    private final static boolean STACKTRACE_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    private final static String STACKTRACE_COLUMN = "stacktrace";

    @Column(name = STACKTRACE_COLUMN, nullable = !STACKTRACE_REQUIRED, columnDefinition = "TEXT")
    private String stacktrace;

    @Required(value = STACKTRACE_REQUIRED)
    @LeadingOrTrailingWhitespace(allowed = STACKTRACE_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getStacktrace()
    {
        return this.stacktrace;
    }

    public void setStacktrace(final String stacktrace)
    {
        this.stacktrace = stacktrace;
    }

    public final static String TIMESTAMP_PROPERTY = "timestamp";

    private final static boolean TIMESTAMP_REQUIRED = false;

    private final static String TIMESTAMP_COLUMN = "timestamp";

    @Column(name = TIMESTAMP_COLUMN, nullable = !TIMESTAMP_REQUIRED)
    // @Temporal(TemporalType.TIMESTAMP)
    private Date timestamp;

    @Required(value = TIMESTAMP_REQUIRED)
    public Date getTimestamp()
    {
        return this.timestamp;
    }

    public void setTimestamp(final Date timestamp)
    {
        this.timestamp = timestamp;
    }

    public final static String VIRTUAL_APP_PROPERTY = "virtualApp";

    private final static boolean VIRTUAL_APP_REQUIRED = false;

    private final static String VIRTUAL_APP_COLUMN = "virtualApp";

    /* package */final static int VIRTUAL_APP_LENGTH_MIN = 0;

    /* package */final static int VIRTUAL_APP_LENGTH_MAX = 255;

    private final static boolean VIRTUAL_APP_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    @Column(name = VIRTUAL_APP_COLUMN, nullable = !VIRTUAL_APP_REQUIRED, length = VIRTUAL_APP_LENGTH_MAX)
    private String virtualApp;

    @Required(value = VIRTUAL_APP_REQUIRED)
    @Length(min = VIRTUAL_APP_LENGTH_MIN, max = VIRTUAL_APP_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = VIRTUAL_APP_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getVirtualApp()
    {
        return this.virtualApp;
    }

    public void setVirtualApp(final String virtualApp)
    {
        this.virtualApp = virtualApp;
    }

    public final static String DATACENTER_PROPERTY = "datacenter";

    private final static boolean DATACENTER_REQUIRED = false;

    private final static String DATACENTER_COLUMN = "datacenter";

    /* package */final static int DATACENTER_LENGTH_MIN = 0;

    /* package */final static int DATACENTER_LENGTH_MAX = 255;

    private final static boolean DATACENTER_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    @Column(name = DATACENTER_COLUMN, nullable = !DATACENTER_REQUIRED, length = DATACENTER_LENGTH_MAX)
    private String datacenter;

    @Required(value = DATACENTER_REQUIRED)
    @Length(min = DATACENTER_LENGTH_MIN, max = DATACENTER_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = DATACENTER_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getDatacenter()
    {
        return this.datacenter;
    }

    public void setDatacenter(final String datacenter)
    {
        this.datacenter = datacenter;
    }

    public final static String VIRTUAL_DATACENTER_PROPERTY = "virtualDatacenter";

    private final static boolean VIRTUAL_DATACENTER_REQUIRED = false;

    private final static String VIRTUAL_DATACENTER_COLUMN = "virtualDataCenter";

    /* package */final static int VIRTUAL_DATACENTER_LENGTH_MIN = 0;

    /* package */final static int VIRTUAL_DATACENTER_LENGTH_MAX = 255;

    private final static boolean VIRTUAL_DATACENTER_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    @Column(name = VIRTUAL_DATACENTER_COLUMN, nullable = !VIRTUAL_DATACENTER_REQUIRED, length = VIRTUAL_DATACENTER_LENGTH_MAX)
    private String virtualDatacenter;

    @Required(value = VIRTUAL_DATACENTER_REQUIRED)
    @Length(min = VIRTUAL_DATACENTER_LENGTH_MIN, max = VIRTUAL_DATACENTER_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = VIRTUAL_DATACENTER_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getVirtualDatacenter()
    {
        return this.virtualDatacenter;
    }

    public void setVirtualDatacenter(final String virtualDatacenter)
    {
        this.virtualDatacenter = virtualDatacenter;
    }

    public final static String ENTERPRISE_PROPERTY = "enterprise";

    private final static boolean ENTERPRISE_REQUIRED = false;

    private final static String ENTERPRISE_COLUMN = "enterprise";

    /* package */final static int ENTERPRISE_LENGTH_MIN = 0;

    /* package */final static int ENTERPRISE_LENGTH_MAX = 255;

    private final static boolean ENTERPRISE_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    @Column(name = ENTERPRISE_COLUMN, nullable = !ENTERPRISE_REQUIRED, length = ENTERPRISE_LENGTH_MAX)
    private String enterprise;

    @Required(value = ENTERPRISE_REQUIRED)
    @Length(min = ENTERPRISE_LENGTH_MIN, max = ENTERPRISE_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = ENTERPRISE_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getEnterprise()
    {
        return this.enterprise;
    }

    public void setEnterprise(final String enterprise)
    {
        this.enterprise = enterprise;
    }

    public final static String STORAGE_SYSTEM_PROPERTY = "storageSystem";

    private final static boolean STORAGE_SYSTEM_REQUIRED = false;

    private final static String STORAGE_SYSTEM_COLUMN = "storageSystem";

    /* package */final static int STORAGE_SYSTEM_LENGTH_MIN = 0;

    /* package */final static int STORAGE_SYSTEM_LENGTH_MAX = 255;

    private final static boolean STORAGE_SYSTEM_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    @Column(name = STORAGE_SYSTEM_COLUMN, nullable = !STORAGE_SYSTEM_REQUIRED, length = STORAGE_SYSTEM_LENGTH_MAX)
    private String storageSystem;

    @Required(value = STORAGE_SYSTEM_REQUIRED)
    @Length(min = STORAGE_SYSTEM_LENGTH_MIN, max = STORAGE_SYSTEM_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = STORAGE_SYSTEM_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getStorageSystem()
    {
        return this.storageSystem;
    }

    public void setStorageSystem(final String storageSystem)
    {
        this.storageSystem = storageSystem;
    }

    public final static String NETWORK_PROPERTY = "network";

    private final static boolean NETWORK_REQUIRED = false;

    private final static String NETWORK_COLUMN = "network";

    /* package */final static int NETWORK_LENGTH_MIN = 0;

    /* package */final static int NETWORK_LENGTH_MAX = 255;

    private final static boolean NETWORK_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    @Column(name = NETWORK_COLUMN, nullable = !NETWORK_REQUIRED, length = NETWORK_LENGTH_MAX)
    private String network;

    @Required(value = NETWORK_REQUIRED)
    @Length(min = NETWORK_LENGTH_MIN, max = NETWORK_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = NETWORK_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getNetwork()
    {
        return this.network;
    }

    public void setNetwork(final String network)
    {
        this.network = network;
    }

    public final static String PHYSICAL_MACHINE_PROPERTY = "physicalMachine";

    private final static boolean PHYSICAL_MACHINE_REQUIRED = false;

    private final static String PHYSICAL_MACHINE_COLUMN = "physicalmachine";

    /* package */final static int PHYSICAL_MACHINE_LENGTH_MIN = 0;

    /* package */final static int PHYSICAL_MACHINE_LENGTH_MAX = 255;

    private final static boolean PHYSICAL_MACHINE_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    @Column(name = PHYSICAL_MACHINE_COLUMN, nullable = !PHYSICAL_MACHINE_REQUIRED, length = PHYSICAL_MACHINE_LENGTH_MAX)
    private String physicalMachine;

    @Required(value = PHYSICAL_MACHINE_REQUIRED)
    @Length(min = PHYSICAL_MACHINE_LENGTH_MIN, max = PHYSICAL_MACHINE_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = PHYSICAL_MACHINE_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getPhysicalMachine()
    {
        return this.physicalMachine;
    }

    public void setPhysicalMachine(final String physicalMachine)
    {
        this.physicalMachine = physicalMachine;
    }

    public final static String RACK_PROPERTY = "rack";

    private final static boolean RACK_REQUIRED = false;

    private final static String RACK_COLUMN = "rack";

    /* package */final static int RACK_LENGTH_MIN = 0;

    /* package */final static int RACK_LENGTH_MAX = 255;

    private final static boolean RACK_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    @Column(name = RACK_COLUMN, nullable = !RACK_REQUIRED, length = RACK_LENGTH_MAX)
    private String rack;

    @Required(value = RACK_REQUIRED)
    @Length(min = RACK_LENGTH_MIN, max = RACK_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = RACK_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getRack()
    {
        return this.rack;
    }

    public void setRack(final String rack)
    {
        this.rack = rack;
    }

    public final static String VIRTUALMACHINE_PROPERTY = "virtualMachine";

    private final static boolean VIRTUALMACHINE_REQUIRED = false;

    private final static String VIRTUALMACHINE_COLUMN = "virtualmachine";

    /* package */final static int VIRTUALMACHINE_LENGTH_MIN = 0;

    /* package */final static int VIRTUALMACHINE_LENGTH_MAX = 255;

    private final static boolean VIRTUALMACHINE_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    @Column(name = VIRTUALMACHINE_COLUMN, nullable = !VIRTUALMACHINE_REQUIRED, length = VIRTUALMACHINE_LENGTH_MAX)
    private String virtualMachine;

    @Required(value = VIRTUALMACHINE_REQUIRED)
    @Length(min = VIRTUALMACHINE_LENGTH_MIN, max = VIRTUALMACHINE_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = VIRTUALMACHINE_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getVirtualMachine()
    {
        return this.virtualMachine;
    }

    public void setVirtualMachine(final String virtualMachine)
    {
        this.virtualMachine = virtualMachine;
    }

    public final static String VOLUME_PROPERTY = "volume";

    private final static boolean VOLUME_REQUIRED = false;

    private final static String VOLUME_COLUMN = "volume";

    /* package */final static int VOLUME_LENGTH_MIN = 0;

    /* package */final static int VOLUME_LENGTH_MAX = 255;

    private final static boolean VOLUME_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    @Column(name = VOLUME_COLUMN, nullable = !VOLUME_REQUIRED, length = VOLUME_LENGTH_MAX)
    private String volume;

    @Required(value = VOLUME_REQUIRED)
    @Length(min = VOLUME_LENGTH_MIN, max = VOLUME_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = VOLUME_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getVolume()
    {
        return this.volume;
    }

    public void setVolume(final String volume)
    {
        this.volume = volume;
    }

    public final static String SUBNET_PROPERTY = "subnet";

    private final static boolean SUBNET_REQUIRED = false;

    private final static String SUBNET_COLUMN = "subnet";

    /* package */final static int SUBNET_LENGTH_MIN = 0;

    /* package */final static int SUBNET_LENGTH_MAX = 255;

    private final static boolean SUBNET_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    @Column(name = SUBNET_COLUMN, nullable = !SUBNET_REQUIRED, length = SUBNET_LENGTH_MAX)
    private String subnet;

    @Required(value = SUBNET_REQUIRED)
    @Length(min = SUBNET_LENGTH_MIN, max = SUBNET_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = SUBNET_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getSubnet()
    {
        return this.subnet;
    }

    public void setSubnet(final String subnet)
    {
        this.subnet = subnet;
    }

    public final static String SEVERITY_PROPERTY = "severity";

    private final static boolean SEVERITY_REQUIRED = true;

    private final static String SEVERITY_COLUMN = "severity";

    private final static int SEVERITY_COLUMN_LENGTH = 100;

    @Enumerated(value = javax.persistence.EnumType.STRING)
    @Column(name = SEVERITY_COLUMN, nullable = !SEVERITY_REQUIRED, length = SEVERITY_COLUMN_LENGTH)
    private SeverityType severity;

    @Required(value = SEVERITY_REQUIRED)
    public SeverityType getSeverity()
    {
        return this.severity;
    }

    public void setSeverity(final SeverityType severity)
    {
        this.severity = severity;
    }

    public final static String USER_PROPERTY = "user";

    private final static boolean USER_REQUIRED = false;

    private final static String USER_COLUMN = "user";

    /* package */final static int USER_LENGTH_MIN = 0;

    /* package */final static int USER_LENGTH_MAX = 255;

    private final static boolean USER_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    @Column(name = USER_COLUMN, nullable = !USER_REQUIRED, length = USER_LENGTH_MAX)
    private String user;

    @Required(value = USER_REQUIRED)
    @Length(min = USER_LENGTH_MIN, max = USER_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = USER_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getUser()
    {
        return this.user;
    }

    public void setUser(final String user)
    {
        this.user = user;
    }

    public final static String ID_USER_PROPERTY = "idUser";

    private final static String ID_USER_COLUMN = "idUser";

    /* package */final static long ID_USER_MIN = 0;

    /* package */final static long ID_USER_MAX = Integer.MAX_VALUE;

    @Column(name = ID_USER_COLUMN, nullable = true)
    @Range(min = ID_USER_MIN, max = ID_USER_MAX)
    private Integer idUser;

    public Integer getIdUser()
    {
        return this.idUser;
    }

    public void setIdUser(final Integer idUser)
    {
        this.idUser = idUser;
    }

    public final static String ID_DATACENTER_PROPERTY = "idDatacenter";

    private final static String ID_DATACENTER_COLUMN = "idDatacenter";

    /* package */final static long ID_DATACENTER_MIN = 0;

    /* package */final static long ID_DATACENTER_MAX = Integer.MAX_VALUE;

    @Column(name = ID_DATACENTER_COLUMN, nullable = true)
    @Range(min = ID_DATACENTER_MIN, max = ID_DATACENTER_MAX)
    private Integer idDatacenter;

    public Integer getIdDatacenter()
    {
        return this.idDatacenter;
    }

    public void setIdDatacenter(final Integer idDatacenter)
    {
        this.idDatacenter = idDatacenter;
    }

    public final static String ID_RACK_PROPERTY = "idRack";

    private final static String ID_RACK_COLUMN = "idRack";

    /* package */final static long ID_RACK_MIN = 0;

    /* package */final static long ID_RACK_MAX = Integer.MAX_VALUE;

    @Column(name = ID_RACK_COLUMN, nullable = true)
    @Range(min = ID_RACK_MIN, max = ID_RACK_MAX)
    private Integer idRack;

    public Integer getIdRack()
    {
        return this.idRack;
    }

    public void setIdRack(final Integer idRack)
    {
        this.idRack = idRack;
    }

    public final static String ID_PHYSICAL_MACHINE_PROPERTY = "idPhysicalMachine";

    private final static String ID_PHYSICAL_MACHINE_COLUMN = "idPhysicalMachine";

    /* package */final static long ID_PHYSICAL_MACHINE_MIN = 0;

    /* package */final static long ID_PHYSICAL_MACHINE_MAX = Integer.MAX_VALUE;

    @Column(name = ID_PHYSICAL_MACHINE_COLUMN, nullable = true)
    @Range(min = ID_PHYSICAL_MACHINE_MIN, max = ID_PHYSICAL_MACHINE_MAX)
    private Integer idPhysicalMachine;

    public Integer getIdPhysicalMachine()
    {
        return this.idPhysicalMachine;
    }

    public void setIdPhysicalMachine(final Integer idPhysicalMachine)
    {
        this.idPhysicalMachine = idPhysicalMachine;
    }

    public final static String ID_STORAGE_SYSTEM_PROPERTY = "idStorageSystem";

    private final static String ID_STORAGE_SYSTEM_COLUMN = "idStorageSystem";

    /* package */final static long ID_STORAGE_SYSTEM_MIN = 0;

    /* package */final static long ID_STORAGE_SYSTEM_MAX = Integer.MAX_VALUE;

    @Column(name = ID_STORAGE_SYSTEM_COLUMN, nullable = true)
    @Range(min = ID_STORAGE_SYSTEM_MIN, max = ID_STORAGE_SYSTEM_MAX)
    private Integer idStorageSystem;

    public Integer getIdStorageSystem()
    {
        return this.idStorageSystem;
    }

    public void setIdStorageSystem(final Integer idStorageSystem)
    {
        this.idStorageSystem = idStorageSystem;
    }

    public final static String ID_STORAGE_POOL_PROPERTY = "idStoragePool";

    private final static boolean ID_STORAGE_POOL_REQUIRED = false;

    private final static String ID_STORAGE_POOL_COLUMN = "idStoragePool";

    /* package */final static int ID_STORAGE_POOL_LENGTH_MIN = 0;

    /* package */final static int ID_STORAGE_POOL_LENGTH_MAX = 255;

    private final static boolean ID_STORAGE_POOL_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    @Column(name = ID_STORAGE_POOL_COLUMN, nullable = !ID_STORAGE_POOL_REQUIRED, length = ID_STORAGE_POOL_LENGTH_MAX)
    private String idStoragePool;

    @Required(value = ID_STORAGE_POOL_REQUIRED)
    @Length(min = ID_STORAGE_POOL_LENGTH_MIN, max = ID_STORAGE_POOL_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = ID_STORAGE_POOL_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getIdStoragePool()
    {
        return this.idStoragePool;
    }

    public void setIdStoragePool(final String idStoragePool)
    {
        this.idStoragePool = idStoragePool;
    }

    public final static String ID_VOLUME_PROPERTY = "idVolume";

    private final static boolean ID_VOLUME_REQUIRED = false;

    private final static String ID_VOLUME_COLUMN = "idVolume";

    /* package */final static int ID_VOLUME_LENGTH_MIN = 0;

    /* package */final static int ID_VOLUME_LENGTH_MAX = 255;

    private final static boolean ID_VOLUME_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    @Column(name = ID_VOLUME_COLUMN, nullable = !ID_VOLUME_REQUIRED, length = ID_VOLUME_LENGTH_MAX)
    private String idVolume;

    @Required(value = ID_VOLUME_REQUIRED)
    @Length(min = ID_VOLUME_LENGTH_MIN, max = ID_VOLUME_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = ID_VOLUME_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getIdVolume()
    {
        return this.idVolume;
    }

    public void setIdVolume(final String idVolume)
    {
        this.idVolume = idVolume;
    }

    public final static String ID_NETWORK_PROPERTY = "idNetwork";

    private final static String ID_NETWORK_COLUMN = "idNetwork";

    /* package */final static long ID_NETWORK_MIN = 0;

    /* package */final static long ID_NETWORK_MAX = Integer.MAX_VALUE;

    @Column(name = ID_NETWORK_COLUMN, nullable = true)
    @Range(min = ID_NETWORK_MIN, max = ID_NETWORK_MAX)
    private Integer idNetwork;

    public Integer getIdNetwork()
    {
        return this.idNetwork;
    }

    public void setIdNetwork(final Integer idNetwork)
    {
        this.idNetwork = idNetwork;
    }

    public final static String ID_SUBNET_PROPERTY = "idSubnet";

    private final static String ID_SUBNET_COLUMN = "idSubnet";

    /* package */final static long ID_SUBNET_MIN = 0;

    /* package */final static long ID_SUBNET_MAX = Integer.MAX_VALUE;

    @Column(name = ID_SUBNET_COLUMN, nullable = true)
    @Range(min = ID_SUBNET_MIN, max = ID_SUBNET_MAX)
    private Integer idSubnet;

    public Integer getIdSubnet()
    {
        return this.idSubnet;
    }

    public void setIdSubnet(final Integer idSubnet)
    {
        this.idSubnet = idSubnet;
    }

    public final static String ID_ENTERPRISE_PROPERTY = "idEnterprise";

    private final static String ID_ENTERPRISE_COLUMN = "idEnterprise";

    /* package */final static long ID_ENTERPRISE_MIN = 0;

    /* package */final static long ID_ENTERPRISE_MAX = Integer.MAX_VALUE;

    @Column(name = ID_ENTERPRISE_COLUMN, nullable = true)
    @Range(min = ID_ENTERPRISE_MIN, max = ID_ENTERPRISE_MAX)
    private Integer idEnterprise;

    public Integer getIdEnterprise()
    {
        return this.idEnterprise;
    }

    public void setIdEnterprise(final Integer idEnterprise)
    {
        this.idEnterprise = idEnterprise;
    }

    public final static String ID_VIRTUALDATACENTER_PROPERTY = "idVirtualDatacenter";

    private final static String ID_VIRTUALDATACENTER_COLUMN = "idVirtualDataCenter";

    /* package */final static long ID_VIRTUALDATACENTER_MIN = 0;

    /* package */final static long ID_VIRTUALDATACENTER_MAX = Integer.MAX_VALUE;

    @Column(name = ID_VIRTUALDATACENTER_COLUMN, nullable = true)
    @Range(min = ID_VIRTUALDATACENTER_MIN, max = ID_VIRTUALDATACENTER_MAX)
    private Integer idVirtualDatacenter;

    public Integer getIdVirtualDatacenter()
    {
        return this.idVirtualDatacenter;
    }

    public void setIdVirtualDatacenter(final Integer idVirtualDataCenter)
    {
        this.idVirtualDatacenter = idVirtualDataCenter;
    }

    public final static String ID_VIRTUAL_APP_PROPERTY = "idVirtualApp";

    private final static String ID_VIRTUAL_APP_COLUMN = "idVirtualApp";

    /* package */final static long ID_VIRTUAL_APP_MIN = 0;

    /* package */final static long ID_VIRTUAL_APP_MAX = Integer.MAX_VALUE;

    @Column(name = ID_VIRTUAL_APP_COLUMN, nullable = true)
    @Range(min = ID_VIRTUAL_APP_MIN, max = ID_VIRTUAL_APP_MAX)
    private Integer idVirtualApp;

    public Integer getIdVirtualApp()
    {
        return this.idVirtualApp;
    }

    public void setIdVirtualApp(final Integer idVirtualApp)
    {
        this.idVirtualApp = idVirtualApp;
    }

    public final static String ID_VIRTUAL_MACHINE_PROPERTY = "idVirtualMachine";

    private final static String ID_VIRTUAL_MACHINE_COLUMN = "idVirtualMachine";

    /* package */final static long ID_VIRTUAL_MACHINE_MIN = 0;

    /* package */final static long ID_VIRTUAL_MACHINE_MAX = Integer.MAX_VALUE;

    @Column(name = ID_VIRTUAL_MACHINE_COLUMN, nullable = true)
    @Range(min = ID_VIRTUAL_MACHINE_MIN, max = ID_VIRTUAL_MACHINE_MAX)
    private Integer idVirtualMachine;

    public Integer getIdVirtualMachine()
    {
        return this.idVirtualMachine;
    }

    public void setIdVirtualMachine(final Integer idVirtualMachine)
    {
        this.idVirtualMachine = idVirtualMachine;
    }
}
