/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.infrastructure;

import java.net.URISyntaxException;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceException;

import org.apache.commons.lang.StringUtils;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.abiquo.model.enumerator.RemoteServiceType;
import com.abiquo.server.core.common.persistence.DefaultDAOBase;

@Repository("jpaRemoteServiceDAO")
public class RemoteServiceDAO extends DefaultDAOBase<Integer, RemoteService>
{
    public RemoteServiceDAO()
    {
        super(RemoteService.class);
    }

    public RemoteServiceDAO(final EntityManager entityManager)
    {
        super(RemoteService.class, entityManager);
    }

    private static Criterion equalUri(final String uri)
    {
        assert !StringUtils.isEmpty(uri);

        return Restrictions.eq(RemoteService.URI_PROPERTY, uri);
    }

    private static Criterion equalDatacenter(final Datacenter datacenter)
    {
        return Restrictions.eq(RemoteService.DATACENTER_PROPERTY, datacenter);
    }

    private static Criterion equalType(final RemoteServiceType type)
    {
        return Restrictions.eq(RemoteService.TYPE_PROPERTY, type);
    }

    public boolean existRemoteServiceUri(final String uri) throws URISyntaxException
    {
        assert !StringUtils.isEmpty(uri);

        return this.existsAnyByCriterions(equalUri(uri));
    }

    public List<RemoteService> findByDatacenter(final Datacenter datacenter)
    {
        return findByCriterions(equalDatacenter(datacenter));
    }

    public List<RemoteService> findByDatacenterAndType(final Datacenter datacenter,
        final RemoteServiceType type)
    {
        return createCriteria(equalDatacenter(datacenter), equalType(type)).list();
    }

    public List<RemoteService> findByType(final RemoteServiceType type)
    {
        return findByCriterions(equalType(type));
    }

    public String getRemoteServiceUri(final Datacenter datacenter, final RemoteServiceType type)
    {
        return getRemoteService(datacenter, type).getUri();
    }

    public RemoteService getRemoteService(final Datacenter datacenter, final RemoteServiceType type)
    {
        List<RemoteService> remoteServices = findByDatacenterAndType(datacenter, type);

        if (remoteServices.size() != 1 || remoteServices.get(0).getStatus() == 0)
        {
            final String cause =
                String.format("Datacenter [%s] doesn't have the ''[%s]'' remote service "
                    + "configured", datacenter.getName(), type.getName());

            throw new PersistenceException(cause); // TODO infrastructure exception
        }

        return remoteServices.get(0);
    }
}
