/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */

package com.abiquo.server.core.infrastructure;

import java.io.Serializable;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Embeddable;
import javax.persistence.EmbeddedId;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

/**
 * 
 * 
 * 
 * @author destevezg
 *
 */
@Entity
@Table(name = ScheduledResources.TABLE_NAME)
@org.hibernate.annotations.Table(appliesTo = ScheduledResources.TABLE_NAME)
public class ScheduledResources
{

    public final static String TABLE_NAME = "scheduled_resources";

    @Embeddable
    public static class Id implements Serializable
    {
        /**
         * 
         */
        private static final long serialVersionUID = 1622028659861358483L;

        @Column(name = "idPhysicalMachine")
        private Integer machineId;

        @Column(name = "idDatastore")
        private Integer datastoreId;

        public Id()
        {
        }

        public Id(final Integer machineId, final Integer datastoreId)
        {
            this.machineId = machineId;
            this.datastoreId = datastoreId;
        }

        @Override
        public int hashCode()
        {
            final int prime = 31;
            int result = 1;
            result = prime * result + (datastoreId == null ? 0 : datastoreId.hashCode());
            result = prime * result + (machineId == null ? 0 : machineId.hashCode());
            return result;
        }

        @Override
        public boolean equals(final Object obj)
        {
            if (this == obj)
            {
                return true;
            }
            if (obj == null)
            {
                return false;
            }
            if (getClass() != obj.getClass())
            {
                return false;
            }
            Id other = (Id) obj;
            if (datastoreId == null)
            {
                if (other.datastoreId != null)
                {
                    return false;
                }
            }
            else if (!datastoreId.equals(other.datastoreId))
            {
                return false;
            }
            if (machineId == null)
            {
                if (other.machineId != null)
                {
                    return false;
                }
            }
            else if (!machineId.equals(other.machineId))
            {
                return false;
            }
            return true;
        }
    }

    @EmbeddedId
    private Id id;

    @Column(name = "cpuUsed", updatable = false)
    private Integer cpuUsed;

    @Column(name = "ramUsed", updatable = false)
    private Integer ramUsed;

    @Column(name = "datastoreUsed", updatable = false)
    private long datastoreUsed;

    @ManyToOne
    @JoinColumn(name = "idPhysicalMachine", nullable = false, insertable = false, updatable = false)
    private Machine machine;

    @ManyToOne(cascade = CascadeType.REMOVE)
    @JoinColumn(name = "idDatastore", nullable = false, insertable = false, updatable = false)
    private Datastore datastore;

    public ScheduledResources()
    {
        super();
        this.id = new Id();
    }

    public ScheduledResources(final Machine machine, final Datastore datastore)
    {
        super();
        this.id = new Id(machine.getId(), datastore.getId());

        this.cpuUsed = 0;
        this.setRamUsed(0);
        this.datastoreUsed = 0;

        this.machine = machine;
        this.datastore = datastore;

        this.machine.getScheduledResources().add(this);
        this.datastore.setScheduledResources(this);
    }

    public Integer getCpuUsed()
    {
        return cpuUsed;
    }

    public void setCpuUsed(final Integer cpuUsed)
    {
        this.cpuUsed = cpuUsed;
    }

    public Machine getMachine()
    {
        return machine;
    }

    public void setMachine(final Machine machine)
    {
        this.machine = machine;
        this.id.machineId = machine.getId();
    }

    public Datastore getDatastore()
    {
        return datastore;
    }

    public void setDatastore(final Datastore datastore)
    {
        this.datastore = datastore;
        this.id.datastoreId = datastore.getId();
    }

    public long getDsUsed()
    {
        return datastoreUsed;
    }

    public void setDsUsed(final Long dsUsed)
    {
        this.datastoreUsed = dsUsed;
    }

    public Integer getRamUsed()
    {
        return ramUsed;
    }

    public void setRamUsed(final Integer ramUsed)
    {
        this.ramUsed = ramUsed;
    }
}
