/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.infrastructure;

import java.util.List;

import javax.persistence.EntityManager;

import org.hibernate.Criteria;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.common.persistence.DefaultDAOBase;

@Repository("jpaUcsRackDAO")
public class UcsRackDAO extends DefaultDAOBase<Integer, UcsRack>
{
    public UcsRackDAO()
    {
        super(UcsRack.class);
    }

    public UcsRackDAO(final EntityManager entityManager)
    {
        super(UcsRack.class, entityManager);
    }

    /**
     * Return all {@links UcsRack} associated to a
     * 
     * @param datacenterId id.
     * @return List<UcsRack> with all {@links UcsRack} associated to the given {@link Datacenter}.
     */
    public List<UcsRack> findAllUcsRacksByDatacenter(final Datacenter datacenter)
    {
        return findAllUcsRacksByDatacenter(datacenter, null);
    }

    public List<UcsRack> findAllUcsRacksByDatacenter(final Datacenter datacenter,
        final String filter)
    {
        Criteria criteria = createCriteria(sameDatacenter(datacenter));

        if (filter != null && !filter.isEmpty())
        {
            criteria.add(filterBy(filter));
        }

        criteria.addOrder(Order.asc(Rack.NAME_PROPERTY));

        return criteria.list();
    }

    /**
     * Criterion with same {@link Datacenter}.
     * 
     * @param datacenterId {@link Datacenter}.
     * @return Criterion
     */
    private Criterion sameDatacenter(final Datacenter datacenterId)
    {
        return Restrictions.eq(Rack.DATACENTER_PROPERTY, datacenterId);
    }

    /**
     * The ip is unique in DB.
     * 
     * @param ip ip.
     * @return boolean
     */
    public boolean existAnyOtherWithIPSameDatacenter(final Datacenter datacenter, final String ip)
    {
        return existsAnyByCriterions(Restrictions.and(Restrictions.eq(UcsRack.IP_PROPERTY, ip),
            sameDatacenter(datacenter)));
    }

    public Criterion filterBy(final String filter)
    {
        Disjunction filterDisjunction = Restrictions.disjunction();

        filterDisjunction.add(Restrictions.like(Rack.NAME_PROPERTY, '%' + filter + '%'));

        return filterDisjunction;
    }

}
