/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.infrastructure.management;

import java.util.List;

import javax.persistence.EntityManager;

import org.hibernate.Criteria;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.cloud.VirtualDatacenter;
import com.abiquo.server.core.cloud.VirtualMachine;
import com.abiquo.server.core.common.persistence.DefaultDAOBase;
import com.abiquo.server.core.common.persistence.JPAConfiguration;
import com.abiquo.server.core.infrastructure.storage.DiskManagement;
import com.abiquo.server.core.infrastructure.storage.VolumeManagement;

@Repository("jpaRasdManagementDAO")
public class RasdManagementDAO extends DefaultDAOBase<Integer, RasdManagement>
{
    public RasdManagementDAO()
    {
        super(RasdManagement.class);
    }

    public RasdManagementDAO(final EntityManager entityManager)
    {
        super(RasdManagement.class, entityManager);
    }

    private static Criterion sameResourceType(final String idResourceType)
    {
        return Restrictions.eq(RasdManagement.ID_RESOURCE_TYPE_PROPERTY, idResourceType);
    }

    private static Criterion sameVirtualDatacenter(final VirtualDatacenter virtualDatacenter)
    {
        return Restrictions.eq(RasdManagement.VIRTUAL_DATACENTER_PROPERTY, virtualDatacenter);
    }

    private static Criterion sameVirtualMachine(final VirtualMachine virtualMachine)
    {
        return Restrictions.eq(RasdManagement.VIRTUAL_MACHINE_PROPERTY, virtualMachine);
    }

    @SuppressWarnings("unchecked")
    public List<RasdManagement> findByVirtualDatacenterAndResourceType(
        final VirtualDatacenter virtualDatacenter, final String resourceType)
    {
        return createCriteria(sameVirtualDatacenter(virtualDatacenter),
            sameResourceType(resourceType)).list();
    }

    public List<RasdManagement> findByVirtualMachine(final VirtualMachine virtualMachine)
    {
        return getResultList(createCriteria(sameVirtualMachine(virtualMachine)));
    }

    public List<RasdManagement> findDisksAndVolumesByVirtualMachine(
        final VirtualMachine virtualMachine)
    {
        Criteria crit = createCriteria();
        crit.createAlias(RasdManagement.RASD_PROPERTY, "rasd");

        // Add disk resource type filter
        crit.add(Restrictions.in(RasdManagement.ID_RESOURCE_TYPE_PROPERTY, new String[] {
        VolumeManagement.DISCRIMINATOR, DiskManagement.DISCRIMINATOR}));

        // Add virtual machine filter
        crit.add(sameVirtualMachine(virtualMachine));

        // Order by generation (attachment order)
        crit.addOrder(Order.asc("sequence"));

        return getResultList(crit);
    }

    public void enableTemporalOnlyFilter()
    {
        JPAConfiguration.enableOnlyTemporalFilters(getEntityManager());
    }

    public void restoreDefaultFilters()
    {
        JPAConfiguration.enableDefaultFilters(getEntityManager());

    }

}
