/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.infrastructure.network;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.hibernate.annotations.ForeignKey;
import org.hibernate.validator.constraints.Length;

import com.abiquo.server.core.common.DefaultEntityBase;
import com.abiquo.server.core.infrastructure.Machine;
import com.softwarementors.validation.constraints.LeadingOrTrailingWhitespace;
import com.softwarementors.validation.constraints.Required;

@Entity
@Table(name = NetworkInterface.TABLE_NAME)
@org.hibernate.annotations.Table(appliesTo = NetworkInterface.TABLE_NAME)
public class NetworkInterface extends DefaultEntityBase
{
    public static final String TABLE_NAME = "network_interface";

    // DO NOT ACCESS: present due to needs of infrastructure support. *NEVER* call from business
    // code
    protected NetworkInterface()
    {
        // Just for JPA support
    }

    public NetworkInterface(final String name, final Machine machine)
    {
        this.name = name;
        this.machine = machine;
    }

    private final static String ID_COLUMN = "id";

    @Id
    @GeneratedValue
    @Column(name = ID_COLUMN, nullable = false)
    private Integer id;

    @Override
    public Integer getId()
    {
        return this.id;
    }

    public final static String NAME_PROPERTY = "name";

    private final static boolean NAME_REQUIRED = true;

    private final static int NAME_LENGTH_MIN = 0;

    private final static int NAME_LENGTH_MAX = 25;

    private final static boolean NAME_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    private final static String NAME_COLUMN = "name";

    @Column(name = NAME_COLUMN, nullable = !NAME_REQUIRED, length = NAME_LENGTH_MAX)
    private String name;

    @Required(value = NAME_REQUIRED)
    @Length(min = NAME_LENGTH_MIN, max = NAME_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = NAME_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getName()
    {
        return this.name;
    }

    public void setName(final String name)
    {
        this.name = name;
    }

    public final static String MAC_PROPERTY = "mac";

    private final static boolean MAC_REQUIRED = false;

    private final static int MAC_LENGTH_MIN = 0;

    private final static int MAC_LENGTH_MAX = 25;

    private final static boolean MAC_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    private final static String MAC_COLUMN = "mac";

    @Column(name = MAC_COLUMN, nullable = !MAC_REQUIRED, length = MAC_LENGTH_MAX)
    private String mac;

    @Required(value = MAC_REQUIRED)
    @Length(min = MAC_LENGTH_MIN, max = MAC_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = MAC_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getMac()
    {
        return this.mac;
    }

    public void setMac(final String mac)
    {
        this.mac = mac;
    }

    public final static String NETWORK_SERVICE_TYPE_PROPERTY = "networkServiceType";

    private final static boolean NETWORK_SERVICE_TYPE_REQUIRED = false;

    private final static String NETWORK_SERVICE_TYPE_ID_COLUMN = "network_service_type_id";

    @JoinColumn(name = NETWORK_SERVICE_TYPE_ID_COLUMN)
    @ManyToOne(fetch = FetchType.LAZY)
    @ForeignKey(name = "FK_" + TABLE_NAME + "_networkServiceType")
    private NetworkServiceType networkServiceType;

    @Required(value = NETWORK_SERVICE_TYPE_REQUIRED)
    public NetworkServiceType getNetworkServiceType()
    {
        return this.networkServiceType;
    }

    public void setNetworkServiceType(final NetworkServiceType networkServiceType)
    {
        this.networkServiceType = networkServiceType;
    }

    public final static String MACHINE_PROPERTY = "machine";

    private final static boolean MACHINE_REQUIRED = true;

    private final static String MACHINE_ID_COLUMN = "physicalmachine_id";

    @JoinColumn(name = MACHINE_ID_COLUMN)
    @ManyToOne(fetch = FetchType.LAZY)
    @ForeignKey(name = "FK_" + TABLE_NAME + "_machine")
    private Machine machine;

    @Required(value = MACHINE_REQUIRED)
    public Machine getMachine()
    {
        return this.machine;
    }

    public void setMachine(final Machine machine)
    {
        this.machine = machine;
    }

    public final static String SHAREDIP_PROPERTY = "sharedIp";

    private final static boolean SHAREDIP_REQUIRED = false;

    private final static int SHAREDIP_LENGTH_MIN = 0;

    private final static int SHAREDIP_LENGTH_MAX = 25;

    private final static boolean SHAREDIP_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    private final static String SHAREDIP_COLUMN = "shared_ip";

    @Column(name = SHAREDIP_COLUMN, nullable = !SHAREDIP_REQUIRED, length = SHAREDIP_LENGTH_MAX)
    private String sharedIp;

    @Required(value = SHAREDIP_REQUIRED)
    @Length(min = SHAREDIP_LENGTH_MIN, max = SHAREDIP_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = SHAREDIP_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getSharedIp()
    {
        return this.sharedIp;
    }

    public void setSharedIp(final String sharedIp)
    {
        this.sharedIp = sharedIp;
    }
}
