/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.infrastructure.network;

import java.util.List;

import javax.persistence.EntityManager;

import org.hibernate.Criteria;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.common.persistence.DefaultDAOBase;
import com.abiquo.server.core.infrastructure.Machine;
import com.softwarementors.bzngine.entities.PersistentEntity;

@Repository("jpaNetworkInterfaceDAO")
public class NetworkInterfaceDAO extends DefaultDAOBase<Integer, NetworkInterface>
{
    public NetworkInterfaceDAO()
    {
        super(NetworkInterface.class);
    }

    public NetworkInterfaceDAO(final EntityManager entityManager)
    {
        super(NetworkInterface.class, entityManager);
    }

    public NetworkInterface findNetworkInterfaceByName(final Machine machine, final String name)
    {
        Criteria criteria = getSession().createCriteria(NetworkInterface.class);
        criteria.add(Restrictions.eq(NetworkInterface.MACHINE_PROPERTY, machine));
        criteria.add(Restrictions.eq(NetworkInterface.NAME_PROPERTY, name));

        return (NetworkInterface) criteria.uniqueResult();
    }

    public NetworkInterface findNetworkInterfaceByServiceType(final Machine machine,
        final NetworkServiceType nst)
    {
        Criteria criteria = getSession().createCriteria(NetworkInterface.class);
        criteria.add(Restrictions.eq(NetworkInterface.MACHINE_PROPERTY, machine));
        criteria.add(Restrictions.eq(NetworkInterface.NETWORK_SERVICE_TYPE_PROPERTY, nst));

        return (NetworkInterface) criteria.uniqueResult();
    }

    public List<NetworkInterface> findListNetworkInterfaceByServiceType(final NetworkServiceType nst)
    {
        Criteria criteria = getSession().createCriteria(NetworkInterface.class);
        criteria.add(Restrictions.eq(NetworkInterface.NETWORK_SERVICE_TYPE_PROPERTY, nst));

        return criteria.list();
    }

    public List<NetworkInterface> findSharedNetworkInterfaceByName(final String sharedIp,
        final String name, final Integer excludedId)
    {
        Criteria criteria = getSession().createCriteria(NetworkInterface.class);
        criteria.add(Restrictions.eq(NetworkInterface.SHAREDIP_PROPERTY, sharedIp));
        criteria.add(Restrictions.eq(NetworkInterface.NAME_PROPERTY, name));
        if (excludedId != null)
        {
            criteria.add(Restrictions.ne(PersistentEntity.ID_PROPERTY, excludedId));
        }
        return criteria.list();
    }

    public Integer findNumberNotAssignedNetworkInterfaces(final Machine machine)
    {
        Criteria criteria = getSession().createCriteria(NetworkInterface.class);
        criteria.add(Restrictions.eq(NetworkInterface.MACHINE_PROPERTY, machine));
        criteria.add(Restrictions.isNull(NetworkInterface.NETWORK_SERVICE_TYPE_PROPERTY));

        return criteria.list().size();
    }

}
