/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.infrastructure.network;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.hibernate.annotations.ForeignKey;
import org.hibernate.validator.constraints.Length;

import com.abiquo.server.core.common.DefaultEntityBase;
import com.abiquo.server.core.infrastructure.Datacenter;
import com.softwarementors.validation.constraints.LeadingOrTrailingWhitespace;
import com.softwarementors.validation.constraints.Required;

@Entity
@Table(name = NetworkServiceType.TABLE_NAME)
@org.hibernate.annotations.Table(appliesTo = NetworkServiceType.TABLE_NAME)
public class NetworkServiceType extends DefaultEntityBase
{
    public static final String TABLE_NAME = "network_service_type";

    // DO NOT ACCESS: present due to needs of infrastructure support. *NEVER* call from business
    // code
    protected NetworkServiceType()
    {
        // Just for JPA support
    }

    public NetworkServiceType(final String name, final Datacenter datacenter,
        final Boolean defaultNST)
    {
        this.name = name;
        this.datacenter = datacenter;
        this.defaultNST = defaultNST;
    }

    private final static String ID_COLUMN = "id";

    @Id
    @GeneratedValue
    @Column(name = ID_COLUMN, nullable = false)
    private Integer id;

    @Override
    public Integer getId()
    {
        return this.id;
    }

    public final static String NAME_PROPERTY = "name";

    private final static boolean NAME_REQUIRED = false;

    private final static int NAME_LENGTH_MIN = 1;

    private final static int NAME_LENGTH_MAX = 50;

    private final static boolean NAME_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    private final static String NAME_COLUMN = "name";

    @Column(name = NAME_COLUMN, nullable = !NAME_REQUIRED, length = NAME_LENGTH_MAX)
    private String name;

    @Required(value = NAME_REQUIRED)
    @Length(min = NAME_LENGTH_MIN, max = NAME_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = NAME_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getName()
    {
        return this.name;
    }

    public void setName(final String name)
    {
        this.name = name;
    }

    public final static String DEFAULT_PROPERTY = "defaultNST";

    private final static String DEFAULT_COLUMN = "default_nst";

    private final static boolean DEFAULT_REQUIRED = true;

    @Column(name = DEFAULT_COLUMN, nullable = false)
    private boolean defaultNST;

    @Required(value = DEFAULT_REQUIRED)
    public boolean getDefaultNST()
    {
        return this.defaultNST;
    }

    public void setDefaultNST(final Boolean defaultNST)
    {
        this.defaultNST = defaultNST;
    }

    public final static String DATACENTER_PROPERTY = "datacenter";

    private final static boolean DATACENTER_REQUIRED = true;

    private final static String DATACENTER_ID_COLUMN = "datacenter_id";

    @JoinColumn(name = DATACENTER_ID_COLUMN)
    @ManyToOne(fetch = FetchType.LAZY)
    @ForeignKey(name = "network_service_type_FK1")
    private Datacenter datacenter;

    @Required(value = DATACENTER_REQUIRED)
    public Datacenter getDatacenter()
    {
        return this.datacenter;
    }

    public void setDatacenter(final Datacenter datacenter)
    {
        this.datacenter = datacenter;
    }

    @Override
    public boolean equals(final Object obj)
    {
        if (obj instanceof NetworkServiceType)
        {
            NetworkServiceType nst = (NetworkServiceType) obj;
            if (this.id != null)
            {
                return this.id.equals(nst.getId()) && this.name.equals(nst.getName());
            }
            return false;
        }
        else
        {
            return false;
        }
    }

}
