/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.infrastructure.network;

import java.util.List;

import javax.persistence.EntityManager;

import org.hibernate.Criteria;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.common.persistence.DefaultDAOBase;
import com.abiquo.server.core.infrastructure.Datacenter;

@Repository("jpaNetworkServiceTypeDAO")
public class NetworkServiceTypeDAO extends DefaultDAOBase<Integer, NetworkServiceType>
{
    public NetworkServiceTypeDAO()
    {
        super(NetworkServiceType.class);
    }

    public NetworkServiceTypeDAO(final EntityManager entityManager)
    {
        super(NetworkServiceType.class, entityManager);
    }

    public List<NetworkServiceType> findByDatacenter(final Datacenter datacenter)
    {
        Criteria criteria = getSession().createCriteria(NetworkServiceType.class);
        criteria.add(Restrictions.eq(NetworkServiceType.DATACENTER_PROPERTY, datacenter));

        return criteria.list();
    }

    public NetworkServiceType findByDatacenterAndId(final Datacenter datacenter, final Integer id)
    {
        Criteria criteria = getSession().createCriteria(NetworkServiceType.class);
        criteria.add(Restrictions.eq(NetworkServiceType.DATACENTER_PROPERTY, datacenter));
        criteria.add(Restrictions.eq("id", id));

        return (NetworkServiceType) criteria.uniqueResult();
    }

    public NetworkServiceType findByDatacenterAndName(final Datacenter datacenter, final String name)
    {
        Criteria criteria = getSession().createCriteria(NetworkServiceType.class);
        criteria.add(Restrictions.eq(NetworkServiceType.DATACENTER_PROPERTY, datacenter));
        criteria.add(Restrictions.eq(NetworkServiceType.NAME_PROPERTY, name));

        return (NetworkServiceType) criteria.uniqueResult();
    }

    public NetworkServiceType findDefaultByDatacenter(final Datacenter datacenter)
    {
        Criteria criteria = getSession().createCriteria(NetworkServiceType.class);
        criteria.add(Restrictions.eq(NetworkServiceType.DATACENTER_PROPERTY, datacenter));
        criteria.add(Restrictions.eq(NetworkServiceType.DEFAULT_PROPERTY, true));

        return (NetworkServiceType) criteria.uniqueResult();
    }

}
