/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.infrastructure.storage;

import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.hibernate.annotations.Filter;
import org.hibernate.annotations.FilterDef;
import org.hibernate.annotations.FilterDefs;
import org.hibernate.annotations.Filters;
import org.hibernate.annotations.ForeignKey;

import com.abiquo.server.core.appslibrary.VirtualMachineTemplate;
import com.abiquo.server.core.cloud.VirtualMachine;
import com.abiquo.server.core.infrastructure.management.RasdManagement;
import com.softwarementors.validation.constraints.Required;

@Entity
@Table(name = DvdManagement.TABLE_NAME)
@DiscriminatorValue(DvdManagement.DISCRIMINATOR)
@FilterDefs({@FilterDef(name = DvdManagement.NOT_TEMP), @FilterDef(name = DvdManagement.ONLY_TEMP)})
@Filters({@Filter(name = DvdManagement.NOT_TEMP, condition = "temporal is null"),
    @Filter(name = DvdManagement.ONLY_TEMP, condition = "temporal is not null")})
public class DvdManagement extends RasdManagement
{
    public static final String DISCRIMINATOR = "16"; // CIMResourceTypeEnum.DVD_drive(16)

    public static final String TABLE_NAME = "dvd_management";

    public static final String NOT_TEMP = "dvdmanagement_not_temp";

    public static final String ONLY_TEMP = "dvdmanagement_only_temp";

    public static final String RESOURCE_NAME = "CD/DVD";

    public static final String RESOURCE_DESCRIPTION = "CD/DVD";

    // DO NOT ACCESS: present due to needs of infrastructure support. *NEVER* call from business
    // code
    public DvdManagement()
    {
        super(DISCRIMINATOR);
    }

    public DvdManagement(final VirtualMachine vm)
    {
        super(DISCRIMINATOR);
        setVirtualMachine(vm);

    }

    public final static String IMAGE_PROPERTY = "image";

    private final static boolean IMAGE_REQUIRED = false;

    private final static String IMAGE_ID_COLUMN = "idImage";

    @JoinColumn(name = IMAGE_ID_COLUMN)
    @ManyToOne(fetch = FetchType.LAZY)
    @ForeignKey(name = "FK_" + TABLE_NAME + "_image")
    private VirtualMachineTemplate image;

    @Required(value = IMAGE_REQUIRED)
    public VirtualMachineTemplate getImage()
    {
        return image;
    }

    public void setImage(final VirtualMachineTemplate image)
    {
        this.image = image;
    }

    @Override
    public void attach(final int sequence, final VirtualMachine vm)
    {
        setVirtualMachine(vm);
        setSequence(sequence);
    }

    @Override
    public void detach()
    {
        // do nothing, dvd doesn't detach.;
        setVirtualMachine(null);
    }

    @Override
    public boolean isAttached()
    {
        return getVirtualMachine() != null;
    }
}
