/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.infrastructure.storage;

import java.util.List;

import javax.persistence.EntityManager;

import org.hibernate.Criteria;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.common.persistence.DefaultDAOBase;

@Repository("jpaStoragePoolDAO")
@SuppressWarnings("unchecked")
/* package */class StoragePoolDAO extends DefaultDAOBase<String, StoragePool>
{
    public StoragePoolDAO()
    {
        super(StoragePool.class);
    }

    public StoragePoolDAO(final EntityManager entityManager)
    {
        super(StoragePool.class, entityManager);
    }

    public List<StoragePool> getPoolsByStorageDevice(final Integer deviceId)
    {
        Criteria criteria = createCriteria(Restrictions.eq("device.id", deviceId));
        return criteria.list();
    }

    public StoragePool findPoolById(final Integer deviceId, final String poolId)
    {
        Criteria criteria =
            createCriteria(Restrictions.eq("device.id", deviceId)).add(
                Restrictions.eq("idStorage", poolId));
        Object obj = criteria.uniqueResult();
        return (StoragePool) obj;
    }

    public StoragePool findPoolByName(final Integer deviceId, final String name)
    {
        Criteria criteria =
            createCriteria(Restrictions.eq("device.id", deviceId)).add(
                Restrictions.eq("name", name));
        Object obj = criteria.uniqueResult();
        return (StoragePool) obj;
    }

    public List<StoragePool> findPoolsByTier(final Tier tier)
    {
        Criteria criteria = createCriteria(Restrictions.eq("tier", tier));
        return criteria.list();
    }

}
