/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.infrastructure.storage;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.hibernate.annotations.ForeignKey;
import org.hibernate.validator.constraints.Length;

import com.abiquo.server.core.common.DefaultEntityBase;
import com.abiquo.server.core.enterprise.AllowedTierDAO;
import com.abiquo.server.core.enterprise.Enterprise;
import com.abiquo.server.core.infrastructure.Datacenter;
import com.softwarementors.validation.constraints.LeadingOrTrailingWhitespace;
import com.softwarementors.validation.constraints.Required;

@Entity
@Table(name = Tier.TABLE_NAME)
@org.hibernate.annotations.Table(appliesTo = Tier.TABLE_NAME)
public class Tier extends DefaultEntityBase
{
    public static final String TABLE_NAME = "tier";

    public Tier()
    {
    }

    /**
     * Public constructor with the mandatory attributes.
     * 
     * @param name name of the tier
     * @param description description of the tier level
     * @param dc datacenter the Tier belongs to.
     */
    public Tier(final String name, final String description, final Datacenter dc)
    {
        this.setName(name);
        this.setDescription(description);
        this.setDatacenter(dc);
        this.setEnabled(Boolean.TRUE);
        this.setDefaultAllowed(Boolean.TRUE);
    }

    /**
     * Public constructor with all attributes. This constructor is needed to get the result of
     * method {@link AllowedTierDAO#getEnabledTiers(Enterprise, Datacenter)} as a list of objects
     * from Tier class
     */
    // FIXME Try to found a best solution that not force the need of this constructor
    public Tier(final Integer id, final String name, final String description, final Datacenter dc,
        final boolean enabled, final boolean defaultAllowed)
    {
        this.id = id;
        this.setName(name);
        this.setDescription(description);
        this.setDatacenter(dc);
        this.setEnabled(enabled);
        this.setDefaultAllowed(defaultAllowed);
    }

    public final static String ID_COLUMN = "id";

    @Id
    @GeneratedValue
    @Column(name = ID_COLUMN, nullable = false)
    private Integer id;

    @Override
    public Integer getId()
    {
        return this.id;
    }

    public final static String NAME_PROPERTY = "name";

    private final static boolean NAME_REQUIRED = true;

    /* package */final static int NAME_LENGTH_MIN = 1;

    /* package */final static int NAME_LENGTH_MAX = 40;

    private final static boolean NAME_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    private final static String NAME_COLUMN = "name";

    @Column(name = NAME_COLUMN, nullable = !NAME_REQUIRED, length = NAME_LENGTH_MAX)
    private String name;

    @Required(value = NAME_REQUIRED)
    @Length(min = NAME_LENGTH_MIN, max = NAME_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = NAME_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getName()
    {
        return this.name;
    }

    public void setName(final String name)
    {
        this.name = name;
    }

    public final static String DATACENTER_PROPERTY = "datacenter";

    private final static boolean DATACENTER_REQUIRED = true;

    private final static String DATACENTER_ID_COLUMN = "idDataCenter";

    @JoinColumn(name = DATACENTER_ID_COLUMN)
    @ManyToOne(fetch = FetchType.LAZY)
    @ForeignKey(name = "FK_" + TABLE_NAME + "_datacenter")
    private Datacenter datacenter;

    @Required(value = DATACENTER_REQUIRED)
    public Datacenter getDatacenter()
    {
        return this.datacenter;
    }

    public void setDatacenter(final Datacenter datacenter)
    {
        this.datacenter = datacenter;
    }

    public final static String ENABLED_PROPERTY = "enabled";

    private final static boolean ENABLED_REQUIRED = true;

    private final static String ENABLED_COLUMN = "isEnabled";

    @Column(name = ENABLED_COLUMN, nullable = !ENABLED_REQUIRED)
    private boolean enabled;

    @Required(value = ENABLED_REQUIRED)
    public boolean getEnabled()
    {
        return this.enabled;
    }

    public void setEnabled(final boolean enabled)
    {
        this.enabled = enabled;
    }

    public final static String DEFAULT_ALLOWED_PROPERTY = "defaultAllowed";

    private final static boolean DEFAULT_ALLOWED_REQUIRED = true;

    private final static String DEFAULT_ALLOWED_COLUMN = "isDefaultAllowed";

    @Column(name = DEFAULT_ALLOWED_COLUMN, nullable = !DEFAULT_ALLOWED_REQUIRED)
    private boolean defaultAllowed;

    @Required(value = DEFAULT_ALLOWED_REQUIRED)
    public boolean getDefaultAllowed()
    {
        return this.defaultAllowed;
    }

    public void setDefaultAllowed(final boolean defaultAllowed)
    {
        this.defaultAllowed = defaultAllowed;
    }

    public final static String DESCRIPTION_PROPERTY = "description";

    private final static boolean DESCRIPTION_REQUIRED = true;

    /* package */final static int DESCRIPTION_LENGTH_MIN = 1;

    /* package */final static int DESCRIPTION_LENGTH_MAX = 255;

    private final static boolean DESCRIPTION_LEADING_OR_TRAILING_WHITESPACES_ALLOWED = false;

    private final static String DESCRIPTION_COLUMN = "description";

    @Column(name = DESCRIPTION_COLUMN, nullable = !DESCRIPTION_REQUIRED, length = DESCRIPTION_LENGTH_MAX)
    private String description;

    @Required(value = DESCRIPTION_REQUIRED)
    @Length(min = DESCRIPTION_LENGTH_MIN, max = DESCRIPTION_LENGTH_MAX)
    @LeadingOrTrailingWhitespace(allowed = DESCRIPTION_LEADING_OR_TRAILING_WHITESPACES_ALLOWED)
    public String getDescription()
    {
        return this.description;
    }

    public void setDescription(final String description)
    {
        this.description = description;
    }

}
