/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.pricing;

import java.util.Collection;
import java.util.List;

import javax.persistence.EntityManager;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.common.persistence.DefaultDAOBase;

@Repository("jpaCosteCodeCurrencyDAO")
public class CostCodeCurrencyDAO extends DefaultDAOBase<Integer, CostCodeCurrency>
{
    public CostCodeCurrencyDAO()
    {
        super(CostCodeCurrency.class);
    }

    public CostCodeCurrencyDAO(final EntityManager entityManager)
    {
        super(CostCodeCurrency.class, entityManager);
    }

    private final static String COST_CODES =
        " select distinct ccc.costCode FROM com.abiquo.server.core.pricing.CostCodeCurrency ccc ";

    private Criterion sameCostCode(final CostCode costCode)
    {
        return Restrictions.eq(CostCodeCurrency.COST_CODE_PROPERTY, costCode);
    }

    private Criterion sameCurrency(final Currency currency)
    {
        return Restrictions.eq(CostCodeCurrency.CURRENCY_PROPERTY, currency);
    }

    public CostCodeCurrency findCurrencyCostCode(final CostCode costCode, final Currency currency)
    {
        return (CostCodeCurrency) createCriteria(sameCostCode(costCode), sameCurrency(currency))
            .uniqueResult();
    }

    public boolean existAnyOtherWithCurrency(final CostCodeCurrency costCodeCurrency,
        final Currency currency, final CostCode costCode)
    {
        return existsAnyOtherByCriterions(costCodeCurrency, sameCurrency(currency),
            sameCostCode(costCode));
    }

    public Collection<CostCodeCurrency> find(final CostCode cc, final Currency currency)
    {

        Criteria criteria = createCriteria();
        if (cc != null)
        {
            criteria.add(sameCostCode(cc));
        }
        if (currency != null)
        {
            criteria.add(sameCurrency(currency));
        }

        return criteria.list();
    }

    @SuppressWarnings("unchecked")
    public List<CostCode> findCostCodesIds()
    {
        Query query = getSession().createQuery(COST_CODES);
        return query.list();
    }

}
