/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.pricing;

import java.util.List;

import javax.persistence.EntityManager;

import org.apache.commons.lang.StringUtils;
import org.hibernate.Criteria;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Disjunction;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.common.persistence.DefaultDAOBase;
import com.abiquo.server.core.util.PagedList;

@Repository("jpaCosteCodeDAO")
public class CostCodeDAO extends DefaultDAOBase<Integer, CostCode>
{
    public CostCodeDAO()
    {
        super(CostCode.class);
    }

    public CostCodeDAO(final EntityManager entityManager)
    {
        super(CostCode.class, entityManager);
    }

    public static Criterion sameName(final String name)
    {
        return Restrictions.eq(PricingTemplate.NAME_PROPERTY, name);
    }

    private Criterion filterBy(final String filter)
    {
        Disjunction filterDisjunction = Restrictions.disjunction();

        filterDisjunction.add(Restrictions.like(CostCode.NAME_PROPERTY, '%' + filter + '%'));
        filterDisjunction.add(Restrictions.like(CostCode.DESCRIPTION_PROPERTY, '%' + filter + '%'));

        return filterDisjunction;
    }

    public boolean existAnyOtherCostCodeWithName(final CostCode costCode, final String name)
    {
        return existsAnyOtherByCriterions(costCode, sameName(name));
    }

    public boolean existAnyOtherCostCodeWithName(final String name)
    {
        return existsAnyByCriterions(sameName(name));
    }

    public List<CostCode> find(final String has, final String orderBy, final boolean desc,
        Integer startwith, Integer numResults)
    {
        Criteria criteria = createCriteria(has, orderBy, desc);

        Integer totalResults = count(criteria).intValue();

        criteria = createCriteria(has, orderBy, desc);

        // Limit 0 means no size filter
        if (numResults == 0)
        {
            numResults = totalResults;
            startwith = 0;
        }
        else if (startwith >= totalResults)
        {
            startwith = totalResults - numResults;
        }
        if (startwith != -1)
        {
            criteria.setFirstResult(startwith);
        }

        criteria.setMaxResults(numResults);

        PagedList<CostCode> list = new PagedList<CostCode>(getResultList(criteria));
        list.setTotalResults(totalResults);
        list.setPageSize(numResults);
        list.setCurrentElement(startwith);

        return list;
    }

    private Criteria createCriteria(final String filter, final String orderBy, final boolean desc)
    {
        Criteria criteria = createCriteria();

        if (!StringUtils.isEmpty(filter))
        {
            criteria.add(filterBy(filter));
        }

        if (!StringUtils.isEmpty(orderBy))
        {
            Order order = Order.asc(orderBy);
            if (desc)
            {
                order = Order.desc(orderBy);
            }
            criteria.addOrder(order);
        }

        return criteria;
    }

}
