/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.pricing;

import java.util.Collection;

import javax.persistence.EntityManager;

import org.hibernate.Criteria;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.common.persistence.DefaultDAOBase;
import com.abiquo.server.core.infrastructure.storage.Tier;

@Repository("jpaPricingTierDAO")
public class PricingTierDAO extends DefaultDAOBase<Integer, PricingTier>
{
    public PricingTierDAO()
    {
        super(PricingTier.class);
    }

    public PricingTierDAO(final EntityManager entityManager)
    {
        super(PricingTier.class, entityManager);
    }

    public Collection<PricingTier> findPricingTiers(final PricingTemplate pricing)
    {
        Criteria criteria = createCriteria(samePricing(pricing));

        return criteria.list();
    }

    public Collection<PricingTier> findPricingTiers(final Tier tier)
    {
        Criteria criteria = createCriteria(sameTier(tier));

        return criteria.list();
    }

    public PricingTier findPricingTier(final Tier tier, final PricingTemplate pricing)
    {
        return (PricingTier) createCriteria(sameTier(tier), samePricing(pricing)).uniqueResult();
    }

    private Criterion sameTier(final Tier tier)
    {
        return Restrictions.eq(PricingTier.TIER_PROPERTY, tier);
    }

    private Criterion samePricing(final PricingTemplate pricing)
    {
        return Restrictions.eq(PricingTier.PRICING_TEMPLATE_PROPERTY, pricing);
    }

    public boolean existAnyOtherWithTier(final PricingTier pt1, final Tier tier,
        final PricingTemplate pricingTemplate)
    {
        return existsAnyOtherByCriterions(pt1, sameTier(tier), samePricing(pricingTemplate));
    }

}
