/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.scheduler;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;

import org.hibernate.annotations.ForeignKey;

import com.abiquo.server.core.common.DefaultEntityBase;
import com.abiquo.server.core.enterprise.Enterprise;
import com.softwarementors.validation.constraints.Required;

@Entity
@Table(name = EnterpriseExclusionRule.TABLE_NAME)
@org.hibernate.annotations.Table(appliesTo = EnterpriseExclusionRule.TABLE_NAME)
@NamedQueries({@NamedQuery(name = "ENTERPRISE_EXCLUSION_RULE.ENTERPRISES_DEPLOYED_IN_HYPERVISOR", query = EnterpriseExclusionRule.ENTERPRISES_DEPLOYED_IN_HYPERVISOR)})
public class EnterpriseExclusionRule extends DefaultEntityBase implements PersistentRule
{
    public static final String TABLE_NAME = "workload_enterprise_exclusion_rule";

    public static final String ENTERPRISES_DEPLOYED_IN_HYPERVISOR = "SELECT vm.enterprise.id "
        + "FROM com.abiquo.server.core.cloud.VirtualMachine vm "
        + "WHERE vm.enterprise.id = :enterprise1 OR vm.enterprise.id = :enterprise2 "
        + "AND vm.hypervisor.id = :hyp_id group by vm.enterprise.id";

    public EnterpriseExclusionRule(final Enterprise enterprise1, final Enterprise enterprise2)
    {
        super();

        setEnterprise1(enterprise1);
        setEnterprise2(enterprise2);
    }

    protected EnterpriseExclusionRule()
    {
    }

    private final static String ID_COLUMN = "id";

    @Id
    @GeneratedValue
    @Column(name = ID_COLUMN, nullable = false)
    private Integer id;

    @Override
    public Integer getId()
    {
        return this.id;
    }

    public final static String ENTERPRISE1_PROPERTY = "enterprise1";

    private final static boolean ENTERPRISE1_REQUIRED = true;

    private final static String ENTERPRISE1_ID_COLUMN = "idEnterprise1";

    @JoinColumn(name = ENTERPRISE1_ID_COLUMN)
    @ManyToOne(fetch = FetchType.LAZY)
    // @Cascade(org.hibernate.annotations.CascadeType.ALL)
    @ForeignKey(name = "FK_" + TABLE_NAME + "_enterprise1")
    private Enterprise enterprise1;

    @Required(value = ENTERPRISE1_REQUIRED)
    public Enterprise getEnterprise1()
    {
        return this.enterprise1;
    }

    public void setEnterprise1(final Enterprise enterprise1)
    {
        this.enterprise1 = enterprise1;
    }

    public final static String ENTERPRISE2_PROPERTY = "enterprise2";

    private final static boolean ENTERPRISE2_REQUIRED = true;

    private final static String ENTERPRISE2_ID_COLUMN = "idEnterprise2";

    @JoinColumn(name = ENTERPRISE2_ID_COLUMN)
    @ManyToOne(fetch = FetchType.LAZY)
    // @Cascade(org.hibernate.annotations.CascadeType.ALL)
    @ForeignKey(name = "FK_" + TABLE_NAME + "_enterprise2")
    private Enterprise enterprise2;

    @Required(value = ENTERPRISE2_REQUIRED)
    public Enterprise getEnterprise2()
    {
        return this.enterprise2;
    }

    public void setEnterprise2(final Enterprise enterprise2)
    {
        this.enterprise2 = enterprise2;
    }

}
