/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.scheduler;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.TypedQuery;

import org.hibernate.Criteria;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.cloud.Hypervisor;
import com.abiquo.server.core.common.persistence.DefaultDAOBase;
import com.abiquo.server.core.enterprise.Enterprise;

@Repository("jpaEnterpriseExclusionRuleDAO")
public class EnterpriseExclusionRuleDAO extends DefaultDAOBase<Integer, EnterpriseExclusionRule>
{
    public EnterpriseExclusionRuleDAO()
    {
        super(EnterpriseExclusionRule.class);
    }

    public EnterpriseExclusionRuleDAO(final EntityManager entityManager)
    {
        super(EnterpriseExclusionRule.class, entityManager);
    }

    public List<EnterpriseExclusionRule> findByEnterprise(final Enterprise enterprise)
    {
        Criteria criteria =
            createCriteria(Restrictions
                .or(sameEnterprise1(enterprise), sameEnterprise2(enterprise)));
        return criteria.list();
    }

    /**
     * Given a {@link EnterpriseExclusionRule}, examines if in the given {@link Hypervisor} the rule
     * is broken.
     * 
     * @param rule rule to check.
     * @param hypervisor hypervisor to check.
     * @return true if both enterprises have machines deployed or false otherwise.
     */
    public boolean isEnterpriseExclusionRuleBroken(final EnterpriseExclusionRule rule,
        final Hypervisor hypervisor)
    {
        TypedQuery<Integer> query =
            getEntityManager().createNamedQuery(
                "ENTERPRISE_EXCLUSION_RULE.ENTERPRISES_DEPLOYED_IN_HYPERVISOR", Integer.class);
        query.setParameter("enterprise1", rule.getEnterprise1().getId());
        query.setParameter("enterprise2", rule.getEnterprise2().getId());
        query.setParameter("hyp_id", hypervisor.getId());

        List<Integer> ids = query.getResultList();
        // If size = 2 then both ids are deployed
        return ids.size() == 2;
    }

    private static Criterion sameEnterprise1(final Enterprise enterprise)
    {
        return Restrictions.eq(EnterpriseExclusionRule.ENTERPRISE1_PROPERTY, enterprise);
    }

    private static Criterion sameEnterprise2(final Enterprise enterprise)
    {
        return Restrictions.eq(EnterpriseExclusionRule.ENTERPRISE2_PROPERTY, enterprise);
    }
}
