/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.scheduler;

import java.io.Serializable;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import com.abiquo.server.core.scheduler.EnterpriseExclusionRule;
import com.abiquo.server.core.scheduler.FitPolicyRule;
import com.abiquo.server.core.scheduler.MachineLoadRule;
import com.abiquo.server.core.scheduler.PersistentRule;

/**
 * Utility methods to manipulate Workload Rules.
 * 
 * @author ibarrera
 */
public class RuleUtils
{
    /**
     * Sorts the given rule list.
     * 
     * @param rules The rule list to sort.
     */
    public static void sort(final List< ? extends PersistentRule> rules)
    {
        Collections.sort(rules, new RuleComparator());
    }

    /**
     * Comparator used to sort a set of Workload rules.
     * 
     * @author ibarrera
     */
    private static class RuleComparator implements Comparator<PersistentRule>, Serializable
    {
        /** Serial UID. */
        private static final long serialVersionUID = 1L;

        @Override
        public int compare(final PersistentRule rule0, final PersistentRule rule1)
        {
            return getWeight(rule0) - getWeight(rule1);
        }

        private int getWeight(final PersistentRule rule)
        {
            if (rule instanceof FitPolicyRule)
            {
                return 0;
            }
            else if (rule instanceof EnterpriseExclusionRule)
            {
                return 1;
            }
            else if (rule instanceof MachineLoadRule)
            {
                return 2;
            }
            else
            {
                return 3;
            }
        }
    }
}
