/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.scheduler;

import com.abiquo.server.core.infrastructure.network.IpPoolManagement;
import com.abiquo.server.core.infrastructure.storage.VolumeManagement;
import java.util.ArrayList;
import java.util.List;

import com.abiquo.server.core.infrastructure.network.NetworkServiceType;

/**
 * Holds the virtual image requirements and other additional virtual machine configuration
 * <ul>
 * <li>{@link VolumeManagement}</li>
 * <li>{@link IpPoolManagement}</li>
 * </ul>
 * 
 * @author apuig
 **/
public class VirtualMachineRequirements
{
    /** Required CPU by the virtual image. */
    protected Long cpu;

    /** Required RAM by the virtual image. TODO units. */
    protected Long ram;

    /** Required HD by the virtual image. TODO units. */
    protected Long hd;

    /** Required space on the Datacenter repository (NFS) virtual image disk file size. TODO units. */
    protected Long repository;

    /** Required space on external storage (all the attached volume size). */
    protected Long storage;

    /** Required public VLAN count. */
    protected Long publicVLAN;

    /** Required public IP count. */
    protected Long publicIP;

    /*
     * List of Network Service types: all the service types a physical machine should have so can
     * attach the network interfaces to the corresponding VLANs.
     */
    protected List<NetworkServiceType> nsts;

    public VirtualMachineRequirements(final Long cpu, final Long ram, final Long hd,
        final Long repository, final Long storage, final Long publicVLAN, final Long publicIP,
        final List<NetworkServiceType> nsts)
    {
        this.cpu = cpu;
        this.ram = ram;
        this.hd = hd;
        this.repository = repository;
        this.storage = storage;
        this.publicVLAN = publicVLAN;
        this.publicIP = publicIP;
        this.nsts = nsts;
    }

    public void addRequirement(final VirtualMachineRequirements addReq)
    {
        this.cpu += addReq.cpu;
        this.ram += addReq.ram;
        this.hd += addReq.hd;
        this.repository += addReq.repository;
        this.storage += addReq.storage;
        this.publicIP += addReq.publicIP;
        this.publicVLAN += addReq.publicVLAN;

        for (NetworkServiceType nst : addReq.nsts)
        {
            if (!this.getNsts().contains(nst))
            {
                this.getNsts().add(nst);
            }
        }
    }
    
    /**
     * Meant for just updating datastores
     * 
     * @param hd
     */
    public VirtualMachineRequirements(final Long hd)
    {
        this.hd = hd;
        
        this.cpu = null;
        this.ram = null;        
        this.repository = null;
        this.storage = null;
        this.publicVLAN = null;
        this.publicIP = null;
    }

    public Long getCpu()
    {
        return cpu;
    }

    public Long getRam()
    {
        return ram;
    }

    public Long getHd()
    {
        return hd;
    }

    public Long getStorage()
    {
        return storage;
    }

    public Long getRepository()
    {
        return repository;
    }

    public Long getPublicVLAN()
    {
        return publicVLAN;
    }

    public Long getPublicIP()
    {
        return publicIP;
    }

    public void setStorage(final Long storage)
    {
        this.storage = storage;
    }

    public void setPublicVLAN(final Long publicVLAN)
    {
        this.publicVLAN = publicVLAN;
    }

    public void setPublicIP(final Long publicIP)
    {
        this.publicIP = publicIP;
    }

    public List<NetworkServiceType> getNsts()
    {
        if (nsts == null)
        {
            nsts = new ArrayList<NetworkServiceType>();
        }
        return nsts;
    }

    public void setNsts(final List<NetworkServiceType> nsts)
    {
        this.nsts = nsts;
    }

}
