/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.statistics;

import java.util.Collection;

import javax.persistence.EntityManager;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import com.abiquo.server.core.common.DefaultRepBase;
import com.abiquo.server.core.enterprise.User;

@Repository
public class StatisticsRep extends DefaultRepBase
{
    @Autowired
    CloudUsageDAO cloudUsageDAO;

    @Autowired
    EnterpriseResourcesDAO enterpriseResourcesDAO;

    @Autowired
    VirtualDatacenterResourcesDAO vdcResourcesDAO;

    @Autowired
    VirtualAppResourcesDAO vappResourcesDAO;

    public StatisticsRep()
    {

    }

    public StatisticsRep(EntityManager em)
    {
        this.entityManager = em;
        this.cloudUsageDAO = new CloudUsageDAO(em);
        this.enterpriseResourcesDAO = new EnterpriseResourcesDAO(em);
        this.vdcResourcesDAO = new VirtualDatacenterResourcesDAO(em);
        this.vappResourcesDAO = new VirtualAppResourcesDAO(em);
    }

    public CloudUsage findCloudUsageByDatacenter(Integer idDatacenter)
    {
        assert idDatacenter != null;

        return this.cloudUsageDAO.calculateCloudUsage(idDatacenter);
    }

    /**
     * Total Cloud Usage has some special cases when showing limits Limits are not total sum of
     * limits by DC. Instead they must show limits defined for all the enterprises, that is
     * CloudUsage with datacenterId = -1
     * 
     * @return
     */
    public CloudUsage findTotalCloudUsage()
    {
        CloudUsage cuTotal = this.cloudUsageDAO.calculateCloudUsage(-1);

        CloudUsage cuLimits = this.cloudUsageDAO.findById(-1);

        if (cuTotal != null && cuLimits != null)
        {
            cuTotal.setPublicIPsReserved(cuLimits.getPublicIPsReserved());
            cuTotal.setStorageReserved(cuLimits.getStorageReserved());
            cuTotal.setVirtualCpuReserved(cuLimits.getVirtualCpuReserved());
            cuTotal.setVirtualMemoryReserved(cuLimits.getVirtualMemoryReserved());
            cuTotal.setVirtualStorageReserved(cuLimits.getVirtualStorageReserved());
        }

        return cuTotal;
    }

    public EnterpriseResources findTotalEnterpriseResources()
    {
        return this.enterpriseResourcesDAO.sumTotalResourcesByEnterprise();
    }

    public EnterpriseResources findResourcesByEnterprise(Integer idEnterprise)
    {
        assert idEnterprise != null;

        return this.enterpriseResourcesDAO.findById(idEnterprise);
    }

    public Collection<VirtualAppResources> findVappResourcesByEnterprise(Integer idEnterprise,
        User user)
    {
        assert idEnterprise != null;

        return this.vappResourcesDAO.findByIdEnterprise(idEnterprise, user);
    }

    public Collection<VirtualDatacenterResources> findVDCResourcesByEnterprise(
        Integer idEnterprise, User user)
    {
        assert idEnterprise != null;

        return this.vdcResourcesDAO.findByIdEnterprise(idEnterprise, user);
    }

    public VirtualDatacenterResources findVDCResourcesById(Integer idVirtualDatacenter)
    {
        assert idVirtualDatacenter != null;

        return this.vdcResourcesDAO.findById(idVirtualDatacenter);
    }

}
