/**
 * Copyright (C) 2008 - Abiquo Holdings S.L. All rights reserved.
 *
 * Please see /opt/abiquo/tomcat/webapps/legal/ on Abiquo server
 * or contact contact@abiquo.com for licensing information.
 */
package com.abiquo.server.core.task;

import static com.abiquo.model.redis.RedisEntityUtils.getEntityKey;

import org.springframework.stereotype.Component;

import redis.clients.jedis.Jedis;
import redis.clients.jedis.Transaction;

import com.abiquo.model.redis.RedisDAOBase;

/**
 * This base class provides Redis-persistence logic for {@link CacheEntry} entity. <h3>Instance to
 * persist</h3>
 * 
 * <pre>
 * job.id = 0
 * job.values = ["A", "B"]
 * </pre>
 * 
 * <h3>Redis structure</h3>
 * 
 * <pre>
 * LPUSH Entry:0 "A"
 * LPUSH Entry:0 "B"
 * </pre>
 * 
 * @author eruiz@abiquo.com
 */
@Component
public class CacheEntryDAO extends RedisDAOBase<CacheEntry>
{
    @Override
    public void delete(CacheEntry entity, Transaction transaction)
    {
        transaction.del(entity.getEntityKey());
    }

    @Override
    public void save(CacheEntry entity, Transaction transaction)
    {
        delete(entity, transaction);

        for (String value : entity.getValues())
        {
            transaction.lpush(entity.getEntityKey(), value);
        }
    }

    public CacheEntry findById(final String id, Jedis jedis)
    {
        String key = getEntityKey(CacheEntry.class, id);
        CacheEntry entry = null;

        if (jedis.exists(key))
        {
            entry = new CacheEntry(id);
            entry.getValues().addAll(jedis.lrange(key, 0, -1));
        }

        return entry;
    }
}
